<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://club.wpeka.com/
 * @since      1.0.0
 *
 * @package    Wpadcenter
 * @subpackage Wpadcenter/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wpadcenter
 * @subpackage Wpadcenter/admin
 * @author     WPEka Club <support@wpeka.com>
 */
class Wpadcenter_Admin {


	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Advertiser instance.
	 *
	 * @since 1.0.1
	 * @access public
	 * @var string $advertiser_obj Advertiser instance.
	 */
	public $advertiser_obj;

	/**
	 * Location string.
	 *
	 * @access private
	 * @var string $location Location string.
	 */
	private $location;

	/** Wpadcenter_Adsense class singleton
	 *
	 * @var \Wpeka\Adcenter\Wpadcenter_Adsense
	 */
	private $adsense;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name The name of this plugin.
	 * @param      string $version The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		$this->adsense     = \Wpeka\Adcenter\Wpadcenter_Adsense::get_instance();

		$this->init_admin_hooks();

	}

	/**
	 * Initialize all admin related hooks like ajax handlers
	 */
	private function init_admin_hooks() {
		add_action( 'acf/render_field/key=field_google_adsense_code', array( $this, 'render_adsense_selection' ), 10 );
		add_action( 'wp_ajax_adsense_load_adcode', array( $this, 'load_google_adsense_code' ), 10 );
	}

	/**
	 * Add custom html to import from google adsense ad type
	 */
	public function render_adsense_selection() {
		$data = $this->adsense->get_ad_units();
		if ( true === $data['error'] ) {
			$url = admin_url( 'admin.php?page=wpadcenter-settings#wpadcenter-adsense' );
			?>
			<a href='<?php echo esc_attr( $url ); ?>'>
				<?php esc_html_e( 'Connect to Adsense', 'wpadcenter' ); ?>
			</a>
			<?php

			return;
		}

		?>
		<?php $nonce = wp_create_nonce( 'wpeka-google-adsense' ); ?>
		<script>
			if ('undefined' == typeof window.AdsenseGAPI) {
				AdsenseGAPI = {};
			}
			AdsenseGAPI.nonce = '<?php echo esc_html( $nonce ); ?>';
		</script>
		<table class="form-table">
			<thead>
			<th><?php esc_html_e( 'Name', 'wpadcenter' ); ?></th>
			<th><?php esc_html_e( 'Status', 'wpadcenter' ); ?></th>
			</thead>
			<tbody id="adsense-adunits">

			<?php foreach ( $data['adunits'] as $unit ) : ?>

				<tr>
					<td><?php echo esc_html( $unit['name'] ); ?></td>
					<td><?php echo esc_html( $unit['status'] ); ?></td>
					<td>
						<button class="button button-primary" data-unitid="<?php echo esc_attr( $unit['id'] ); ?>"
								id="<?php echo esc_attr( $unit['code'] ); ?>">
							<?php esc_html_e( 'Load', 'wpadcenter' ); ?>
						</button>
					</td>
				</tr>
			<?php endforeach; ?>
			</tbody>
		</table>

		<?php

		wp_enqueue_script(
			$this->plugin_name . 'ad-selection-script',
			plugin_dir_url( __FILE__ ) . 'js/wpadcenter-ad-selection' . WPADCENTER_SCRIPT_SUFFIX . '.js',
			array( 'jquery' ),
			$this->version,
			true
		);

	}

	/**
	 * Fetch adcode from the api
	 */
	public function load_google_adsense_code() {
		$nonce = isset( $_POST['_wpnonce'] ) ? sanitize_text_field( wp_unslash( $_POST['_wpnonce'] ) ) : '';
		if ( ! isset( $_POST['adunit'] ) || false === wp_verify_nonce( $nonce, 'wpeka-google-adsense' ) ) {
			wp_send_json(
				array(
					'error'   => true,
					'message' => 'Invalid Nonce',
				)
			);
		}

		$code = $this->adsense->get_ad_code( sanitize_text_field( wp_unslash( $_POST['adunit'] ) ) );
		wp_send_json( $code );

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_register_style(
			$this->plugin_name . '-settings',
			plugin_dir_url(
				__FILE__
			) . 'css/wpadcenter-admin-settings' . WPADCENTER_SCRIPT_SUFFIX . '.css',
			array(),
			$this->version,
			'all'
		);
		wp_register_style(
			$this->plugin_name . '-getting-started',
			plugin_dir_url(
				__FILE__
			) . 'css/wpadcenter-admin-getting-started' . WPADCENTER_SCRIPT_SUFFIX . '.css',
			array(),
			$this->version,
			'all'
		);
		wp_register_style(
			$this->plugin_name . '-go-pro',
			plugin_dir_url( __FILE__ ) . 'css/wpadcenter-admin-go-pro' . WPADCENTER_SCRIPT_SUFFIX . '.css',
			array(),
			$this->version,
			'all'
		);
		wp_register_style(
			$this->plugin_name . '-getting-started-wizard',
			plugin_dir_url(
				__FILE__
			) . 'css/wpadcenter-admin-getting-started-wizard' . WPADCENTER_SCRIPT_SUFFIX . '.css',
			array(),
			$this->version,
			'all'
		);
		wp_register_style(
			$this->plugin_name . '-vue-form-generator',
			plugin_dir_url( __FILE__ ) . 'js/libraries/vue-form-wizard/dist/vfg.css',
			array(),
			$this->version,
			'all'
		);
		wp_register_style(
			$this->plugin_name . '-vue-form-wizard',
			plugin_dir_url( __FILE__ ) . 'js/libraries/vue-form-wizard/dist/vue-form-wizard.min.css',
			array(),
			$this->version,
			'all'
		);

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		$wpadcenter_cpts = array( 'wpadcenter-adzones', 'wpadcenter-campaigns', 'wpadcenter-banners' );
		if ( in_array( get_post_type(), $wpadcenter_cpts, true ) ) {
			wp_dequeue_script( 'autosave' );
		}
		wp_enqueue_script(
			$this->plugin_name . 'gapi-settings',
			plugin_dir_url( __FILE__ ) . 'js/wpadcenter-gapi-settings' . WPADCENTER_SCRIPT_SUFFIX . '.js',
			array( 'jquery' ),
			$this->version,
			true
		);
		wp_register_script(
			$this->plugin_name . '-settings',
			plugin_dir_url( __FILE__ ) . 'js/wpadcenter-admin-settings' . WPADCENTER_SCRIPT_SUFFIX . '.js',
			array( 'jquery' ),
			$this->version,
			false
		);
		wp_register_script(
			$this->plugin_name . '-getting-started-wizard',
			plugin_dir_url( __FILE__ ) . 'js/libraries/wpadcenter-admin-getting-started-wizard.js',
			array( 'jquery' ),
			$this->version,
			true
		);
		wp_register_script(
			$this->plugin_name . '-vue',
			plugin_dir_url( __FILE__ ) . 'js/libraries/vue-form-wizard/vue.min.js',
			array(),
			$this->version,
			true
		);
		wp_register_script(
			$this->plugin_name . '-vuelidate',
			plugin_dir_url( __FILE__ ) . 'js/libraries/vue-form-wizard/dist/vuelidate.min.js',
			array(),
			$this->version,
			true
		);
		wp_register_script(
			$this->plugin_name . '-vuelidators',
			plugin_dir_url( __FILE__ ) . 'js/libraries/vue-form-wizard/dist/validators.min.js',
			array(),
			$this->version,
			true
		);
		wp_register_script(
			$this->plugin_name . '-vue-form-wizard',
			plugin_dir_url( __FILE__ ) . 'js/libraries/vue-form-wizard/dist/vue-form-wizard.js',
			array(),
			$this->version,
			true
		);
	}

	/**
	 * Register the JavaScript for the ACF fields.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_enqueue_acf_scripts() {
		wp_enqueue_script(
			$this->plugin_name . '-acf',
			plugin_dir_url( __FILE__ ) . 'js/wpadcenter-admin-acf' . WPADCENTER_SCRIPT_SUFFIX . '.js',
			array( 'jquery' ),
			$this->version,
			false
		);
	}

	/**
	 * Adds action links to the plugin list table.
	 *
	 * Fired by `plugin_action_links` filter.
	 *
	 * @since 1.0.0
	 *
	 * @param array $links An array of plugin action links.
	 * @return array An array of plugin action links.
	 */
	public function wpadcenter_plugin_action_links( $links ) {
		if ( ! get_option( 'wpadcenter_pro_installed' ) ) {
			$links['go_pro'] = sprintf(
				'<a href="%1$s" target="_blank" class="wpadcenter-gopro">%2$s</a>',
				'https://club.wpeka.com/latest-plugins/wpadcenter/',
				__( 'Go Pro', 'wpadcenter' )
			);
		}
		return $links;
	}

	/**
	 * Set screen options.
	 *
	 * @param string $status status.
	 * @param string $option option.
	 * @param string $value value.
	 * @return mixed
	 */
	public function wpadcenter_set_screen_option( $status, $option, $value ) {
		return $value;
	}

	/**
	 * Advertiser screen options.
	 */
	public function wpadcenter_advertisers_screen_option() {
		$option = 'per_page';
		$args   = array(
			'label'   => __( 'Advertisers', 'wpadcenter' ),
			'default' => 10,
			'option'  => 'advertisers_per_page',
		);

		add_screen_option( $option, $args );

		$this->advertiser_obj = new Wpadcenter_Advertiser();
	}

	/**
	 * Register the menu for the plugin.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_admin_menu() {
		// Create Menu for WPAdCenter.
		add_menu_page(
			'WPAdCenter',
			'WPAdCenter',
			'manage_options',
			'wpadcenter-settings',
			array( $this, 'wpadcenter_settings' ),
			WPADCENTER_PLUGIN_URL . 'images/menu-icon.png'
		);
		add_submenu_page(
			'wpadcenter-settings',
			'Settings',
			__( 'Settings', 'wpadcenter' ),
			'manage_options',
			'wpadcenter-settings',
			array( $this, 'wpadcenter_settings' )
		);
		add_submenu_page(
			'wpadcenter-settings',
			'Ad Zones',
			__( 'Ad Zones', 'wpadcenter' ),
			'manage_options',
			'edit.php?post_type=wpadcenter-adzones'
		);
		add_submenu_page(
			'wpadcenter-settings',
			'Advertisers',
			__( 'Advertisers', 'wpadcenter' ),
			'manage_options',
			'wpadcenter-advertisers',
			array( $this, 'wpadcenter_advertisers_list_table' )
		);
		add_submenu_page(
			'wpadcenter-settings',
			'Campaigns',
			__( 'Campaigns', 'wpadcenter' ),
			'manage_options',
			'edit.php?post_type=wpadcenter-campaigns'
		);
		add_submenu_page(
			'wpadcenter-settings',
			'Ads',
			__( 'Ads', 'wpadcenter' ),
			'manage_options',
			'edit.php?post_type=wpadcenter-banners'
		);
		do_action( 'wpadcenter_admin_menu', 'wpadcenter-settings', 'manage_options' );
		add_submenu_page(
			'wpadcenter-settings',
			'Getting Started Wizard',
			__( 'Ad Wizard', 'wpadcenter' ),
			'manage_options',
			'wpadcenter-getting-started-wizard',
			array( $this, 'wpadcenter_getting_started_wizard' )
		);
		add_submenu_page(
			'wpadcenter-settings',
			'Getting Started',
			__( 'Getting Started', 'wpadcenter' ),
			'manage_options',
			'wpadcenter-getting-started',
			array( $this, 'wpadcenter_getting_started' )
		);
		if ( ! get_option( 'wpadcenter_pro_active' ) ) {
			add_submenu_page(
				'wpadcenter-settings',
				'Go Pro',
				__( 'Go Pro', 'wpadcenter' ),
				'manage_options',
				'wpadcenter-go-pro',
				array( $this, 'wpadcenter_go_pro' )
			);
		}
	}

	/**
	 * Update settings.
	 *
	 * @since 1.0.1
	 */
	public function wpadcenter_admin_init() {
		if ( ! wp_next_scheduled( 'wpadcenter_daily_cron' ) ) {
			wp_schedule_event( strtotime( 'tomorrow' ), 'daily', 'wpadcenter_daily_cron' );
		}
		// Update settings from Version 1.0.1.
		$prev_options = get_option( 'WPAdCenter-1.0' );
		if ( isset( $prev_options ) && is_array( $prev_options ) ) {
			$prev_options['enable_scripts']     = false;
			$prev_options['header_scripts']     = '';
			$prev_options['body_scripts']       = '';
			$prev_options['footer_scripts']     = '';
			$prev_options['enable_advertisers'] = false;
			unset( $prev_options['notification_email'] );
			unset( $prev_options['notification_name'] );
			unset( $prev_options['paypal'] );
			unset( $prev_options['paypal_email'] );
			unset( $prev_options['paypal_currency'] );
			unset( $prev_options['paypal_sandbox'] );
			unset( $prev_options['stripe'] );
			unset( $prev_options['stripe_secret_key'] );
			unset( $prev_options['stripe_publisher_key'] );
			unset( $prev_options['stripe_currency'] );
			update_option( WPADCENTER_SETTINGS_FIELD, $prev_options );
			delete_option( 'WPAdCenter-1.0' );
		}
		// Update setting from Version 1.0.3.
		$prev_options = get_option( 'WPAdCenter-2.0' );
		if ( isset( $prev_options ) && is_array( $prev_options ) ) {
			$prev_options['trim_stats'] = '0';
			update_option( WPADCENTER_SETTINGS_FIELD, $prev_options );
			delete_option( 'WPAdCenter-2.0' );
		}
		// Update setting from Version 1.0.4.
		$prev_options = get_option( 'WPAdCenter-3.0' );
		if ( isset( $prev_options ) && is_array( $prev_options ) ) {
			$prev_options['enable_ads_txt']  = false;
			$prev_options['ads_txt_content'] = '';
			update_option( WPADCENTER_SETTINGS_FIELD, $prev_options );
			delete_option( 'WPAdCenter-3.0' );
		}
		add_option( 'wpadcenter_active', true );
	}

	/**
	 * Add advertiser role to the users.
	 */
	public function wpadcenter_init() {
		// Dummy gettext calls to get strings in the catalog.
		/* translators: user role */
		_x( 'Advertiser', 'User role', 'wpadcenter' );

		// Customer role.
		add_role(
			'advertiser',
			'Advertiser',
			array(
				'read' => true,
			)
		);
	}

	/**
	 * Daily schedule campaigns cron.
	 *
	 * @since 1.0.1
	 */
	public function wpadcenter_daily_schedule_campaigns() {
		$the_options  = Wpadcenter::wpadcenter_get_settings();
		$notification = $the_options['notification'];
		$today        = gmdate( 'd-m-Y' );
		$args         = array(
			'post_type'   => 'wpadcenter-campaigns',
			'post_status' => array( 'adc-running', 'adc-scheduled' ),
			'numberposts' => -1,
		);
		$campaigns    = get_posts( $args );
		if ( $campaigns ) {
			foreach ( $campaigns as $campaign ) {
				$start_date       = str_replace( '/', '-', get_field( 'start_date', $campaign->ID ) );
				$end_date         = str_replace( '/', '-', get_field( 'end_date', $campaign->ID ) );
				$campaigns_status = get_field( 'campaigns_status', $campaign->ID );
				$flag             = false;
				if ( strtotime( $today ) > strtotime( $end_date ) ) {
					$campaigns_status = 'finished';
					$flag             = true;
					$end_email_sent   = get_post_meta( $campaign->ID, '_end_email_sent', true );
					if ( '1' !== $end_email_sent ) {
						if ( get_option( 'wpadcenter_pro_active' ) && $notification ) {
							do_action( 'woocommerce_adcenter_campaign_end', $campaign->ID );
							update_post_meta( $campaign->ID, '_end_email_sent', '1' );
						}
					}
				} elseif ( strtotime( $today ) <= strtotime( $start_date ) ) {
					$campaigns_status = 'running';
					$flag             = true;
					$start_email_sent = get_post_meta( $campaign->ID, '_start_email_sent', true );
					if ( '1' !== $start_email_sent ) {
						if ( get_option( 'wpadcenter_pro_active' ) && $notification ) {
							do_action( 'woocommerce_adcenter_campaign_start', $campaign->ID );
							update_post_meta( $campaign->ID, '_start_email_sent', '1' );
						}
					}
				}
				if ( $flag ) {
					update_field( 'campaigns_status', $campaigns_status, $campaign->ID );
					remove_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );
					wp_update_post(
						array(
							'ID'          => $campaign->ID,
							'post_status' => 'adc-' . trim( $campaigns_status ),
						)
					);
					add_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );
				}
			}
		}
	}

	/**
	 * Register WPAdCenter widget.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_widget() {
		register_widget( 'Wpadcenter_Widget' );
	}

	/**
	 * Callback function for Settings menu.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_settings() {
		// Lock out non-admins.
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_attr__( 'You do not have sufficient permission to perform this operation', 'wpadcenter' ) );
		}
		wp_enqueue_style( $this->plugin_name . '-settings' );
		wp_enqueue_script( $this->plugin_name . '-settings' );
		$the_options = Wpadcenter::wpadcenter_get_settings();
		// Check if form has been set.
		if ( isset( $_POST['update_admin_settings_form'] ) || ( isset( $_POST['wpadcenter_settings_ajax_update'] ) ) ) {
			// Check nonce.
			check_admin_referer( 'wpadcenter-update-' . WPADCENTER_SETTINGS_FIELD );
			if ( 'update_admin_settings_form' === $_POST['wpadcenter_settings_ajax_update'] ) {
				foreach ( $the_options as $key => $value ) {
					if ( 'ads_txt_content' === $key ) {
						continue;
					}
					if ( isset( $_POST[ $key . '_field' ] ) ) {
						// Store sanitised values only.
						$the_options[ $key ] = Wpadcenter::wpadcenter_sanitise_settings(
							$key,
							wp_unslash(
								$_POST[ $key . '_field' ]
							)
						); // phpcs:ignore input var ok, CSRF ok, sanitization ok.
					}
				}
				$the_options = apply_filters( 'wpadcenter_after_save_settings', $the_options );
				update_option( WPADCENTER_SETTINGS_FIELD, $the_options );
				echo '<div class="updated"><p><strong>' . esc_attr__(
					'Settings Updated.',
					'wpadcenter'
				) . '</strong></p></div>';
			}
		}

		require_once plugin_dir_path( __FILE__ ) . 'partials/wpadcenter-admin-display.php';
	}

	/**
	 * Process ads.txt content.
	 *
	 * @param Array $the_options setting options.
	 * @return mixed
	 */
	public function wpadcenter_after_save_settings( $the_options ) {
		if ( isset( $_POST['ads_txt_tab'] ) ) {
			check_admin_referer( 'wpadcenter-update-' . WPADCENTER_SETTINGS_FIELD );
			$ads_txt_tab = isset( $_POST['ads_txt_tab'] ) ? sanitize_text_field( wp_unslash( $_POST['ads_txt_tab'] ) ) : '0';
			if ( '1' === $ads_txt_tab && isset( $the_options['enable_ads_txt'] ) ) {
				// process ads.txt content.
				$ads_txt_content = isset( $_POST['ads_txt_content_field'] ) ? esc_textarea(
					$_POST['ads_txt_content_field']
				) : ''; // phpcs:ignore input var ok, CSRF ok, sanitization ok.
				if ( isset( $ads_txt_content ) ) {
					$ads_txt_content                = explode( "\n", $ads_txt_content );
					$ads_txt_content                = array_filter( array_map( 'trim', $ads_txt_content ) );
					$ads_txt_content                = implode( "\n", $ads_txt_content );
					$the_options['ads_txt_content'] = $ads_txt_content;
				}
			}
		}
		return $the_options;
	}

	/**
	 * Check for ads.txt problems.
	 */
	public function wpadcenter_check_ads_txt_problems() {
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_attr__( 'You do not have sufficient permission to perform this operation', 'wpadcenter' ) );
		}
		if ( isset( $_POST['action'] ) ) {
			check_admin_referer( 'check_ads_txt_problems', 'security' );
		}
		$notices = $this->wpadcenter_get_notices();
		echo wp_json_encode( $notices );
		exit();
	}

	/**
	 * Check for existing ads.txt file replace.
	 */
	public function wpadcenter_check_ads_txt_replace() {
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_attr__( 'You do not have sufficient permission to perform this operation', 'wpadcenter' ) );
		}
		if ( isset( $_POST['action'] ) ) {
			check_admin_referer( 'check_ads_txt_replace', 'security' );
		}
		$action_notices = array();
		$remove         = $this->wpadcenter_ads_txt_replace();
		if ( is_wp_error( $remove ) ) {
			$action_notices['response']              = false;
			$action_notices['replace_error_message'] = $remove->get_error_message();
		} else {
			$the_options                     = Wpadcenter::wpadcenter_get_settings();
			$action_notices['response']      = true;
			$action_notices['file_imported'] = __( 'The ads.txt is now managed with WPAdCenter.', 'wpadcenter' );
			$action_notices['file_content']  = $the_options['ads_txt_content'];
		}

		$notices = $this->wpadcenter_get_notices();
		$notices = array_merge( $notices, $action_notices );
		echo wp_json_encode( $notices );
		exit();
	}

	/**
	 * Replace existing ads.txt file.
	 *
	 * @return bool|string|WP_Error
	 */
	public function wpadcenter_ads_txt_replace() {
		if ( ! is_super_admin() ) {
			new WP_Error( 'not_main_site', __( 'Not the main blog', 'wpadcenter' ) );
		}
		$fs_connect = '';
		if ( is_wp_error( $this->wpadcenter_fs_connect() ) ) {
			return $fs_connect;
		}
		global $wp_filesystem;
		$abspath = trailingslashit( $wp_filesystem->abspath() );
		$file    = $abspath . 'ads.txt';
		if ( $wp_filesystem->exists( $file ) && $wp_filesystem->is_file( $file ) ) {
			$the_options    = Wpadcenter::wpadcenter_get_settings();
			$data           = $wp_filesystem->get_contents( $file );
			$file_records   = $this->wpadcenter_parse_file( $data );
			$plugin_records = $this->wpadcenter_parse_file( $the_options['ads_txt_content'] );
			foreach ( $file_records as $k => $record ) {
				foreach ( $plugin_records as $r ) {
					if ( $record[0] === $r[0] ) {
						unset( $file_records[ $k ] );
					}
				}
			}
			$file_records = array_merge( $file_records, $plugin_records );
			$r            = '';
			foreach ( $file_records as $rec ) {
				if ( ! empty( $rec[1] ) ) {
					foreach ( $rec[1] as $rec1 ) {
						$r .= $rec1 . "\n";
					}
				}
				$r .= $rec[0] . "\n";
			}
			$the_options['ads_txt_content'] = $r;
			update_option( WPADCENTER_SETTINGS_FIELD, $the_options );
			if ( $wp_filesystem->delete( $file ) ) {
				return true;
			} else {
				return new WP_Error( 'could_not_delete', __( 'Could not delete the existing ads.txt file', 'wpadcenter' ) );
			}
		} else {
			return new WP_Error( 'not_found', __( 'Could not find the existing ads.txt file', 'wpadcenter' ) );
		}
	}

	/**
	 * Parse file data.
	 *
	 * @param Object $file File.
	 * @return array
	 */
	public function wpadcenter_parse_file( $file ) {
		$lines    = preg_split( '/\r\n|\r|\n/', $file );
		$comments = array();
		$records  = array();
		foreach ( $lines as $line ) {
			$line    = explode( '#', $line );
			$comment = trim( $line[1] );
			if ( ! empty( $line[1] ) && $comment ) {
				$comments[] = '# ' . $comment;
			}

			if ( ! trim( $line[0] ) ) {
				continue;
			}

			$rec  = explode( ',', $line[0] );
			$data = array();

			foreach ( $rec as $k => $r ) {
				$r = trim( $r, " \n\r\t," );
				if ( $r ) {
					$data[] = $r;
				}
			}

			if ( $data ) {
				// Add the record and comments that were placed above or to the right of it.
				$records[] = array( implode( ', ', $data ), $comments );
			}

			$comments = array();
		}
		return $records;
	}

	/**
	 * File connect function.
	 *
	 * @return bool|WP_Error
	 */
	public function wpadcenter_fs_connect() {
		global $wp_filesystem;
		$fs_connect = Wpadcenter::fs_connect( array( ABSPATH ) );

		if ( false === $fs_connect || is_wp_error( $fs_connect ) ) {
			$message = __( 'Unable to connect to the filesystem. Please confirm your credentials.', 'wpadcenter' );

			if ( $wp_filesystem instanceof WP_Filesystem_Base && is_wp_error(
				$wp_filesystem->errors
			) && $wp_filesystem->errors->get_error_code() ) {
				$message = esc_html( $wp_filesystem->errors->get_error_message() );
			}
			if ( is_wp_error( $fs_connect ) && $fs_connect->get_error_code() ) {
				$message = esc_html( $fs_connect->get_error_message() );
			}
			return new WP_Error( 'can_not_connect', $message );
		}
		return true;
	}

	/**
	 * Get ads.txt notices.
	 *
	 * @return array
	 */
	public function wpadcenter_get_notices() {
		$notices     = array(
			'response'      => false,
			'error_message' => esc_html__( 'Something went wrong.', 'wpadcenter' ),
		);
		$the_options = Wpadcenter::wpadcenter_get_settings();
		$url         = home_url( '/' );
		$parsed_url  = wp_parse_url( $url );
		if ( ! isset( $parsed_url['scheme'] ) || ! isset( $parsed_url['host'] ) ) {
			return $notices;
		}
		$link = sprintf( '<a href="%1$s" target="_blank">%1$s</a>', esc_url( $url . 'ads.txt' ) );
		if ( $this->wpadcenter_is_subdir() ) {
			$notices['error_message'] = sprintf(
				/* translators: %s main site file path */
				esc_html__(
					'The ads.txt file cannot be placed because the URL contains a subdirectory. You need to make the file available at %s',
					'wpadcenter'
				),
				sprintf(
					'<a href="%1$s" target="_blank">%1$s</a>',
					esc_url( $parsed_url['scheme'] . '://' . $parsed_url['host'] )
				)
			);
		} else {
			$notices['error_message'] = '';
			$file                     = $this->wpadcenter_get_file_info( $url );
			if ( ! is_wp_error( $file ) ) {
				$notices['response'] = true;
				if ( $file['exists'] ) {
					$notices['file_available'] = '<p>' . sprintf(
						/* translators: %s file path */
						esc_html__( 'The file is available on %s.', 'wpadcenter' ),
						$link
					) . '</p>';
				} else {
					$notices['file_available'] = '<p>' . esc_html__( 'The file was not created.', 'wpadcenter' ) . '</p>';
				}

				if ( $file['is_third_party'] ) {
					$message = sprintf(
						/* translators: %s third party file path */
						esc_html__( 'A third-party file exists: %s', 'wpadcenter' ),
						$link
					);

					if ( is_super_admin() ) {
						$button   = ' <input type="button" class="button" style="vertical-align: middle;" name="replace_ads_txt_file" value="%s" />';
						$message .= sprintf( $button, __( 'Import & Replace', 'wpadcenter' ) );
						$message .= '<p class="wpadcenter_form_help">'
							. __(
								'Move the content of the existing ads.txt file into WPAdCenter and remove it.',
								'wpadcenter'
							)
							. '</p>';
					}
					$notices['is_third_party'] = $message;
				}
			} else {
				$notices['error_message'] = sprintf(
					/* translators: %s default file errors */
					esc_html__( 'An error occured: %s.', 'wpadcenter' ),
					esc_html( $file->get_error_message() )
				);
			}
			if ( $this->wpadcenter_get_root_domain_info() ) {
				$notices['domain_error_message'] = '<p>' . sprintf(
					/* translators: %s the line that may need to be added manually */
					esc_html__(
						'If your site is located on a subdomain, you need to add the following line to the ads.txt file of the root domain: %s',
						'wpadcenter'
					),
					// Without http://.
						'<code>subdomain=' . esc_html( $parsed_url['host'] ) . '</code>'
				) . '</p>';
			}
		}
		return $notices;
	}

	/**
	 * Get root domain info.
	 *
	 * @param null $url URL.
	 * @return bool
	 */
	public function wpadcenter_get_root_domain_info( $url = null ) {
		$url        = $url ? $url : home_url( '/' );
		$parsed_url = wp_parse_url( $url );
		if ( ! isset( $parsed_url['host'] ) ) {
			return false;
		}
		$host = $parsed_url['host'];
		if ( WP_Http::is_ip_address( $host ) ) {
			return false;
		}
		$host_parts = explode( '.', $host );
		$count      = count( $host_parts );
		if ( $count < 3 ) {
			return false;
		}
		if ( 3 === $count ) {
			// Example: `http://one.{net/org/gov/edu/co}.two`.
			$suffixes = array( 'net', 'org', 'gov', 'edu', 'co' );
			if ( in_array( $host_parts[ $count - 2 ], $suffixes, true ) ) {
				return false;
			}
			// Example: `one.com.au'.
			$suffix_and_tld = implode( '.', array_slice( $host_parts, 1 ) );
			if ( in_array( $suffix_and_tld, array( 'com.au', 'com.br', 'com.pl' ), true ) ) {
				return false;
			}
			// `http://www.one.two` will only be crawled if `http://one.two` redirects to it.
			// Check if such redirect exists.
			if ( 'www' === $host_parts[0] ) {
				/*
				 * Do not append `/ads.txt` because otherwise the redirect will not happen.
				 */
				$no_www_url = $parsed_url['scheme'] . '://' . trailingslashit( $host_parts[1] . '.' . $host_parts[2] );

				add_action( 'requests-requests.before_redirect', array( $this, 'collect_locations' ) );
				wp_remote_get(
					$no_www_url,
					array(
						'timeout'     => 5,
						'redirection' => 3,
					)
				);
				remove_action( 'requests-requests.before_redirect', array( $this, 'collect_locations' ) );

				$no_www_url_parsed = wp_parse_url( $this->location );
				if ( isset( $no_www_url_parsed['host'] ) && $no_www_url_parsed['host'] === $host ) {
					return false;
				}
			}
		}
		return true;
	}

	/**
	 * Set location.
	 *
	 * @param string $location Location.
	 */
	public function wpadcenter_collect_locations( $location ) {
		$this->location = $location;
	}

	/**
	 * Get File info.
	 *
	 * @param null $url URL.
	 * @return array|WP_Error
	 */
	public function wpadcenter_get_file_info( $url = null ) {
		$url = $url ? $url : home_url( '/' );

		// Disable ssl verification to prevent errors on servers that are not properly configured with its https certificates.
		/** This filter is documented in wp-includes/class-wp-http-streams.php */
		$sslverify    = apply_filters( 'https_local_ssl_verify', false );
		$response     = wp_remote_get(
			trailingslashit( $url ) . 'ads.txt',
			array(
				'timeout'   => 3,
				'sslverify' => $sslverify,
				'headers'   => array(
					'Cache-Control' => 'no-cache',
				),
			)
		);
		$code         = wp_remote_retrieve_response_code( $response );
		$content      = wp_remote_retrieve_body( $response );
		$content_type = wp_remote_retrieve_header( $response, 'content-type' );

		if ( is_wp_error( $response ) ) {
			return $response;
		}

		$file_exists   = ! is_wp_error( $response ) && 404 !== $code && ( false !== stripos( $content_type, 'text/plain' ) );
		$header_exists = false !== strpos( $content, Wpadcenter::TOP );

		$res = array(
			'exists'         => $file_exists && $header_exists,
			'is_third_party' => $file_exists && ! $header_exists,
		);
		return $res;
	}

	/**
	 * Check if si sub-directory.
	 *
	 * @param null $url URL.
	 * @return bool
	 */
	public function wpadcenter_is_subdir( $url = null ) {
		$url        = $url ? $url : home_url( '/' );
		$parsed_url = wp_parse_url( $url );
		if ( ! empty( $parsed_url['path'] ) && '/' !== $parsed_url['path'] ) {
			return true;
		}
		return false;

	}

	/**
	 * Callback function for advertiser page.
	 */
	public function wpadcenter_advertisers_list_table() {
		?>
		<div class="wrap">
			<h2>Advertisers</h2>
			<div id="poststuff">
				<div id="post-body" class="metabox-holder">
					<div id="post-body-content">
						<div class="meta-box-sortables ui-sortable">
							<form method="post">
								<?php
								$this->advertiser_obj->prepare_items();
								$this->advertiser_obj->search_box( 'Search Advertiser', 'advertiser' );
								$this->advertiser_obj->display();
								?>
							</form>
						</div>
					</div>
				</div>
				<br class="clear">
			</div>
		</div>
		<?php
	}

	/**
	 * Callback function for Go Pro.
	 *
	 * @since 1.0.3
	 */
	public function wpadcenter_go_pro() {
		wp_enqueue_style( $this->plugin_name . '-go-pro' );
		require_once plugin_dir_path( __FILE__ ) . 'views/admin-display-go-pro.php';
	}

	/**
	 * Callback function for Wizard menu.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_getting_started_wizard() {
		// Lock out non-admins.
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_attr__( 'You do not have sufficient permission to perform this operation', 'wpadcenter' ) );
		}
		wp_enqueue_style( $this->plugin_name . '-vue-form-wizard' );
		wp_enqueue_style( $this->plugin_name . '-getting-started-wizard' );
		wp_enqueue_media();
		wp_enqueue_script( $this->plugin_name . '-vue' );
		wp_enqueue_script( $this->plugin_name . '-vuelidate' );
		wp_enqueue_script( $this->plugin_name . '-vuelidators' );
		wp_enqueue_script( $this->plugin_name . '-vue-form-wizard' );
		wp_enqueue_script( $this->plugin_name . '-getting-started-wizard' );
		wp_localize_script(
			$this->plugin_name . '-getting-started-wizard',
			'ajax_url',
			array(
				'admin'    => admin_url(),
				'url'      => admin_url( 'admin-ajax.php' ),
				'security' => wp_create_nonce( 'getting_started_wizard' ),
			)
		);
		$ads_data = $this->adsense->get_ad_units();
		require_once plugin_dir_path( __FILE__ ) . 'views/admin-display-getting-started-wizard.php';
	}

	/**
	 * Deliberately allows class and ID declarations to assist with custom CSS styling.
	 * Returns list of HTML tags allowed in HTML fields for use in declaration of wp_kset field validation.
	 *
	 * @return array
	 */
	public function wpadcenter_allowed_html() {
		$allowed_html = array(
			// Allowed:		<a href="" id="" class="" title="" target="">...</a>.
			// Not allowed:	<a href="javascript(...);">...</a>.
			'a'      => array(
				'href'   => array(),
				'id'     => array(),
				'class'  => array(),
				'title'  => array(),
				'target' => array(),
				'rel'    => array(),
				'style'  => array(),
				'data'   => array(),
			),
			'b'      => array(),
			'br'     => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'div'    => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'em'     => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'i'      => array(),
			'img'    => array(
				'src'   => array(),
				'id'    => array(),
				'class' => array(),
				'alt'   => array(),
				'style' => array(),
			),
			'p'      => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'span'   => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'strong' => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
			'label'  => array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			),
		);
		$html5_tags   = array( 'article', 'section', 'aside', 'details', 'figcaption', 'figure', 'footer', 'header', 'main', 'mark', 'nav', 'summary', 'time' );
		foreach ( $html5_tags as $html5_tag ) {
			$allowed_html[ $html5_tag ] = array(
				'id'    => array(),
				'class' => array(),
				'style' => array(),
			);
		}
		return $allowed_html;
	}

	/**
	 * Returns list of allowed protocols, for use in declaration of wp_kset field validation.
	 * N.B. JavaScript is specifically disallowed for security reasons.
	 * Don't even trust your own database, as you don't know if another plugin has written to your settings.
	 *
	 * @return array
	 */
	public function wpadcenter_allowed_protocols() {
		// Additional options : ftp, ftps, mailto, news, irc, gopher, nntp, feed, telnet.
		return array( 'http', 'https' );
	}

	/**
	 * Returns sanitised content based on field-specific rules defined here
	 * used for both read AND write operations.
	 *
	 * @param string $key Key.
	 * @param string $value Value.
	 * @return bool|string
	 */
	public function wpadcenter_sanitize_adzone_fields( $key, $value ) {
		switch ( $key ) {
			case 'adzone_name':
			case 'adzone_template':
			case 'adzone_custom_width':
			case 'adzone_custom_height':
			case 'transition_effect':
			case 'transition_delay':
			case 'transition_speed':
				$value = sanitize_text_field( $value );
				break;
			case 'adzone_description':
				$value = wp_kses( $value, $this->wpadcenter_allowed_html(), $this->wpadcenter_allowed_protocols() );
				break;
			case 'auto_refresh':
				if ( 'true' === $value || true === $value ) {
					$value = true;
				} else {
					$value = false;
				}
				break;
		}
		return $value;
	}

	/**
	 * Returns sanitised content based on field-specific rules defined here
	 * used for both read AND write operations.
	 *
	 * @param string $key Key.
	 * @param string $value Value.
	 * @return bool|string
	 */
	public function wpadcenter_sanitize_advertiser_fields( $key, $value ) {
		switch ( $key ) {
			case 'advertiser_name':
				$value = sanitize_text_field( $value );
				break;
			case 'advertiser_email':
				$value = sanitize_email( $value );
				break;
		}
		return $value;
	}

	/**
	 * Returns sanitised content based on field-specific rules defined here
	 * used for both read AND write operations.
	 *
	 * @param string $key Key.
	 * @param string $value Value.
	 * @return bool|string
	 */
	public function wpadcenter_sanitize_campaign_fields( $key, $value ) {
		switch ( $key ) {
			case 'campaign_name':
			case 'campaign_advertiser':
			case 'campaign_start':
			case 'campaign_end':
				$value = sanitize_text_field( $value );
				break;
		}
		return $value;
	}

	/**
	 * Returns sanitised content based on field-specific rules defined here
	 * used for both read AND write operations.
	 *
	 * @param string $key Key.
	 * @param string $value Value.
	 * @return bool|string
	 */
	public function wpadcenter_sanitize_ad_fields( $key, $value ) {
		switch ( $key ) {
			case 'ad_name':
			case 'ad_type':
			case 'ad_advertiser':
			case 'ad_campaign':
			case 'ad_adzone':
			case 'ad_image':
				$value = sanitize_text_field( $value );
				break;
			case 'ad_link_url':
			case 'ad_image_link_url':
				$value = sanitize_trackback_urls( $value );
				break;
			case 'ad_link_target':
			case 'ad_link_nofollow':
				if ( 'true' === $value || true === $value ) {
					$value = true;
				} else {
					$value = false;
				}
				break;
		}
		return $value;
	}

	/**
	 * Save wizard step data.
	 */
	public function wpadcenter_save_wizard_data() {
		$response = array(
			'success' => false,
			'id'      => '0',
		);
		if ( ! isset( $_POST['request'] ) ) {
			wp_send_json( $response );
		}
		$request  = isset( $_POST['request'] ) ? sanitize_text_field( wp_unslash( $_POST['request'] ) ) : '';
		$security = isset( $_POST['security'] ) ? sanitize_text_field( wp_unslash( $_POST['security'] ) ) : '';
		if ( ! wp_verify_nonce( $security, 'getting_started_wizard' ) ) {
			wp_send_json( $response );
		}

		switch ( $request ) {
			case 'save_adzone':
				$adzone_data = array();
				if ( isset( $_POST['data'] ) ) {
					foreach ( $_POST['data'] as $key => $value ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.
						if ( isset( $_POST['data'][ $key ] ) && is_array(
							$_POST['data'][ $key ]
						) ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.
							continue;
						} else {
							$adzone_data[ $key ] = $this->wpadcenter_sanitize_adzone_fields(
								$key,
								wp_unslash( $value )
							); // phpcs:ignore input var ok, CSRF ok, sanitization ok.
						}
					}
				}
				$args      = array(
					'post_type'   => 'wpadcenter-adzones',
					'post_status' => 'publish',
					'post_title'  => isset( $adzone_data['adzone_name'] ) ? $adzone_data['adzone_name'] : '',
				);
				$adzone_id = wp_insert_post( $args );
				if ( ! is_wp_error( $adzone_id ) ) {
					update_field( 'adzones_description', $adzone_data['adzone_description'], $adzone_id );
					update_field( 'template', $adzone_data['adzone_template'], $adzone_id );
					update_field( 'custom_width', $adzone_data['adzone_custom_width'], $adzone_id );
					update_field( 'custom_height', $adzone_data['adzone_custom_height'], $adzone_id );
					update_field( 'auto_refresh_ads', $adzone_data['auto_refresh'], $adzone_id );
					update_field( 'transition_effect', $adzone_data['transition_effect'], $adzone_id );
					update_field( 'transition_speed', $adzone_data['transition_speed'], $adzone_id );
					update_field( 'transition_delay', $adzone_data['transition_delay'], $adzone_id );
					$response['success'] = true;
					$response['id']      = $adzone_id;
				}
				break;
			case 'save_advertiser':
				$advertiser_data = array();
				if ( isset( $_POST['data'] ) ) {
					foreach ( $_POST['data'] as $key => $value ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.
						if ( isset( $_POST['data'][ $key ] ) && is_array(
							$_POST['data'][ $key ]
						) ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.
							continue;
						} else {
							$advertiser_data[ $key ] = $this->wpadcenter_sanitize_advertiser_fields(
								$key,
								wp_unslash(
									$value
								)
							); // phpcs:ignore input var ok, CSRF ok, sanitization ok.
						}
					}
				}
				$email        = $advertiser_data['advertiser_email'];
				$login        = strstr( $email, '@', true );
				$pass         = wp_generate_password( 12, false );
				$display_name = $advertiser_data['advertiser_name'];
				if ( email_exists( $email ) ) {
					$user    = get_user_by( 'email', $email );
					$user_id = $user->ID;
				} else {
					$args    = array(
						'user_login'   => $login,
						'user_pass'    => $pass,
						'user_email'   => $email,
						'display_name' => $display_name,
						'first_name'   => $display_name,
						'role'         => 'advertiser',
					);
					$user_id = wp_insert_user( $args );
					if ( is_wp_error( $user_id ) ) {
						$user_id = 0;
					}
				}
				$response['success'] = true;
				$response['id']      = $user_id;
				break;
			case 'save_campaign':
				$campaign_data = array();
				if ( isset( $_POST['data'] ) ) {
					foreach ( $_POST['data'] as $key => $value ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.
						if ( isset( $_POST['data'][ $key ] ) && is_array(
							$_POST['data'][ $key ]
						) ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.
							continue;
						} else {
							$campaign_data[ $key ] = $this->wpadcenter_sanitize_campaign_fields(
								$key,
								wp_unslash( $value )
							); // phpcs:ignore input var ok, CSRF ok, sanitization ok.
						}
					}
				}
				if ( $campaign_data['campaign_start'] > strtotime( gmdate( 'd-m-Y' ) ) ) {
					$campaign_status = 'scheduled';
				} else {
					$campaign_status = 'running';
				}
				$args        = array(
					'post_type'   => 'wpadcenter-campaigns',
					'post_status' => 'adc-' . trim( $campaign_status ),
					'post_title'  => $campaign_data['campaign_name'],
				);
				$campaign_id = wp_insert_post( $args );
				if ( ! is_wp_error( $campaign_id ) ) {
					$campaign_start_date = gmdate( 'Ymd', strtotime( $campaign_data['campaign_start'] ) );
					$campaign_end_date   = gmdate( 'Ymd', strtotime( $campaign_data['campaign_end'] ) );
					update_field( 'campaigns_status', $campaign_status, $campaign_id );
					update_field( 'start_date', $campaign_start_date, $campaign_id );
					update_field( 'end_date', $campaign_end_date, $campaign_id );
					update_field( 'campaigns_advertiser', $campaign_data['campaign_advertiser'], $campaign_id );
					$response['success'] = true;
					$response['id']      = $campaign_id;
				}
				break;
			case 'save_ad':
				$ads_data = array();
				if ( isset( $_POST['data'] ) ) {
					foreach ( $_POST['data'] as $key => $value ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.
						if ( is_array(
							$_POST['data'][ $key ]
						) ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.
							continue;
						} else {
							$ads_data[ $key ] = $this->wpadcenter_sanitize_ad_fields(
								$key,
								wp_unslash( $value )
							); // phpcs:ignore input var ok, CSRF ok, sanitization ok.
						}
					}
				}
				$args  = array(
					'post_type'   => 'wpadcenter-banners',
					'post_status' => 'publish',
					'post_title'  => $ads_data['ad_name'],
				);
				$ad_id = wp_insert_post( $args );
				if ( ! is_wp_error( $ad_id ) ) {
					update_field( 'link_url', $ads_data['ad_link_url'], $ad_id );
					update_field( 'link_target', $ads_data['ad_link_target'], $ad_id );
					update_field( 'link_nofollow', $ads_data['ad_link_nofollow'], $ad_id );
					update_field( 'banner_type', $ads_data['ad_type'], $ad_id );
					update_field( 'banners_advertiser', $ads_data['ad_advertiser'], $ad_id );
					update_field( 'banners_campaign', $ads_data['ad_campaign'], $ad_id );
					update_field( 'banners_ad_zone', $ads_data['ad_adzone'], $ad_id );
					switch ( $ads_data['ad_type'] ) {
						case 'image':
							update_field( 'image', $ads_data['ad_image'], $ad_id );
							break;
						case 'external_image_link':
							update_field( 'image_link', $ads_data['ad_image_link_url'], $ad_id );
							break;
						case 'ad_code':
							update_field( 'ad_code', $ads_data['ad_code'], $ad_id );
							break;
						case 'adsense_code':
							update_field( 'field_google_adsense_code', $ads_data['google_adsense_code'], $ad_id );
							break;
					}
					$response['success'] = true;
					$response['id']      = $ad_id;
				}
				break;
		}

		echo wp_json_encode( $response );
		exit;
	}

	/**
	 * Get wizard step data.
	 */
	public function wpadcenter_get_wizard_data() {
		$response = array();
		if ( isset( $_POST['request'] ) ) {
			$request  = isset( $_POST['request'] ) ? sanitize_text_field( wp_unslash( $_POST['request'] ) ) : '';
			$security = isset( $_POST['security'] ) ? sanitize_text_field( wp_unslash( $_POST['security'] ) ) : '';
			$id       = isset( $_POST['id'] ) ? sanitize_text_field( wp_unslash( $_POST['id'] ) ) : '0';
			if ( wp_verify_nonce( $security, 'getting_started_wizard' ) ) {
				switch ( $request ) {
					case 'get_adzones':
					case 'get_ad_adzones':
						$data = $this->wpadcenter_get_adzones();
						if ( ! empty( $data ) ) {
							foreach ( $data as $row ) {
								$response[] = array(
									'id'   => $row->ID,
									'name' => $row->post_title,
								);
							}
						}
						if ( 'get_adzones' === $request ) {
							$response[] = array(
								'id'   => '0',
								'name' => __( 'Create Ad Zone', 'wpadcenter' ),
							);
						}
						break;
					case 'get_advertisers':
					case 'get_campaign_advertisers':
					case 'get_ad_advertisers':
						$data = $this->wpadcenter_get_advertisers();
						if ( ! empty( $data ) ) {
							foreach ( $data as $row ) {
								$response[] = array(
									'id'   => $row->ID,
									'name' => $row->display_name,
								);
							}
						}
						if ( 'get_advertisers' === $request ) {
							$response[] = array(
								'id'   => '0',
								'name' => __( 'Create Advertiser', 'wpadcenter' ),
							);
						}
						break;
					case 'get_campaigns':
					case 'get_ad_campaigns':
						$data = $this->wpadcenter_get_campaigns( $id );
						if ( ! empty( $data ) ) {
							foreach ( $data as $row ) {
								$response[] = array(
									'id'   => $row->ID,
									'name' => $row->post_title,
								);
							}
						}
						if ( 'get_campaigns' === $request ) {
							$response[] = array(
								'id'   => '0',
								'name' => __( 'Create Campaign', 'wpadcenter' ),
							);
						}
						break;
				}
			}
		}
		echo wp_json_encode( $response );
		exit;
	}

	/**
	 * Get Adzones.
	 *
	 * @return int[]|WP_Post[]
	 */
	public function wpadcenter_get_adzones() {
		$args    = array(
			'post_type'   => 'wpadcenter-adzones',
			'post_status' => 'publish',
			'numberposts' => -1,
		);
		$adzones = get_posts( $args );
		return $adzones;
	}

	/**
	 * Get Advertisers.
	 *
	 * @return array
	 */
	public function wpadcenter_get_advertisers() {
		$args        = array(
			'role'   => 'advertiser',
			'number' => -1,
		);
		$advertisers = get_users( $args );
		return $advertisers;
	}

	/**
	 * Get Campaigns.
	 *
	 * @param string $id Advertiser ID.
	 * @return int[]|WP_Post[]
	 */
	public function wpadcenter_get_campaigns( $id = '0' ) {
		$args = array(
			'post_type'   => 'wpadcenter-campaigns',
			'post_status' => array( 'adc-running', 'adc-scheduled' ),
			'numberposts' => -1,
		);
		if ( '0' !== $id ) {
			$args['meta_query'] = array( // phpcs:ignore slow query
				array(
					'key'     => 'campaigns_advertiser',
					'value'   => $id,
					'compare' => '=',
				),
			);
		}
		$campaigns = get_posts( $args );
		return $campaigns;
	}

	/**
	 * Callback function for Getting Started menu.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_getting_started() {
		// Lock out non-admins.
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_attr__( 'You do not have sufficient permission to perform this operation', 'wpadcenter' ) );
		}
		wp_enqueue_style( $this->plugin_name . '-getting-started' );
		require_once plugin_dir_path( __FILE__ ) . 'views/admin-display-getting-started.php';
	}

	/**
	 * Define CPT's.
	 *
	 * @since 1.0.0
	 * @return mixed|void
	 */
	public function wpadcenter_get_cpt_args() {
		$cpt_args              = array();
		$cpt_args['adzones']   = apply_filters(
			'wpadcenter_cpt_args_adzones',
			array(
				'labels'              => apply_filters(
					'wpadcenter_cpt_args_labels_adzones',
					array(
						'name'               => __( 'Ad Zones', 'wpadcenter' ),
						'singular_name'      => __( 'Ad Zone', 'wpadcenter' ),
						'menu_name'          => __( 'Ad Zones', 'wpadcenter' ),
						'all_items'          => __( 'All Ad Zones', 'wpadcenter' ),
						'add_new_item'       => __( 'Add New Ad Zone', 'wpadcenter' ),
						'add_new'            => __( 'Add New', 'wpadcenter' ),
						'new_item'           => __( 'New Ad Zone', 'wpadcenter' ),
						'edit_item'          => __( 'Edit Ad Zone', 'wpadcenter' ),
						'update_item'        => __( 'Update Ad Zone', 'wpadcenter' ),
						'view_item'          => __( 'View Ad Zone', 'wpadcenter' ),
						'view_items'         => __( 'View Ad Zones', 'wpadcenter' ),
						'search_items'       => __( 'Search Ad Zone', 'wpadcenter' ),
						'not_found'          => __( 'Not found', 'wpadcenter' ),
						'not_found_in_trash' => __( 'Not found in Trash', 'wpadcenter' ),
					)
				),
				'supports'            => array( 'title' ),
				'public'              => true,
				'show_ui'             => true,
				'show_in_menu'        => false,
				'show_in_admin_bar'   => false,
				'show_in_nav_menus'   => false,
				'can_export'          => true,
				'has_archive'         => false,
				'hierarchical'        => false,
				'exclude_from_search' => true,
				'show_in_rest'        => false,
				'publicly_queryable'  => false,
				'rewrite'             => array( 'slug' => 'wpadcenter-adzones' ),
				'capability_type'     => 'post',
			)
		);
		$cpt_args['campaigns'] = apply_filters(
			'wpadcenter_cpt_args_campaigns',
			array(
				'labels'              => apply_filters(
					'wpadcenter_cpt_args_labels_campaigns',
					array(
						'name'               => __( 'Campaigns', 'wpadcenter' ),
						'singular_name'      => __( 'Campaign', 'wpadcenter' ),
						'menu_name'          => __( 'Campaigns', 'wpadcenter' ),
						'all_items'          => __( 'All Campaigns', 'wpadcenter' ),
						'add_new_item'       => __( 'Add New Campaign', 'wpadcenter' ),
						'add_new'            => __( 'Add New', 'wpadcenter' ),
						'new_item'           => __( 'New Campaign', 'wpadcenter' ),
						'edit_item'          => __( 'Edit Campaign', 'wpadcenter' ),
						'update_item'        => __( 'Update Campaign', 'wpadcenter' ),
						'view_item'          => __( 'View Campaign', 'wpadcenter' ),
						'view_items'         => __( 'View Campaigns', 'wpadcenter' ),
						'search_items'       => __( 'Search Campaign', 'wpadcenter' ),
						'not_found'          => __( 'Not found', 'wpadcenter' ),
						'not_found_in_trash' => __( 'Not found in Trash', 'wpadcenter' ),
					)
				),
				'supports'            => array( 'title' ),
				'public'              => true,
				'show_ui'             => true,
				'show_in_menu'        => false,
				'show_in_admin_bar'   => false,
				'show_in_nav_menus'   => false,
				'can_export'          => true,
				'has_archive'         => false,
				'hierarchical'        => false,
				'exclude_from_search' => true,
				'show_in_rest'        => false,
				'publicly_queryable'  => false,
				'rewrite'             => array( 'slug' => 'wpadcenter-campaigns' ),
				'capability_type'     => 'post',
			)
		);
		$cpt_args['banners']   = apply_filters(
			'wpadcenter_cpt_args_banners',
			array(
				'labels'              => apply_filters(
					'wpadcenter_cpt_args_labels_banners',
					array(
						'name'               => __( 'Ads', 'wpadcenter' ),
						'singular_name'      => __( 'Ad', 'wpadcenter' ),
						'menu_name'          => __( 'Ads', 'wpadcenter' ),
						'all_items'          => __( 'All Ads', 'wpadcenter' ),
						'add_new_item'       => __( 'Add New Ad', 'wpadcenter' ),
						'add_new'            => __( 'Add New', 'wpadcenter' ),
						'new_item'           => __( 'New Ad', 'wpadcenter' ),
						'edit_item'          => __( 'Edit Ad', 'wpadcenter' ),
						'update_item'        => __( 'Update Ad', 'wpadcenter' ),
						'view_item'          => __( 'View Ad', 'wpadcenter' ),
						'view_items'         => __( 'View Ads', 'wpadcenter' ),
						'search_items'       => __( 'Search Ad', 'wpadcenter' ),
						'not_found'          => __( 'Not found', 'wpadcenter' ),
						'not_found_in_trash' => __( 'Not found in Trash', 'wpadcenter' ),
					)
				),
				'supports'            => array( 'title' ),
				'public'              => true,
				'show_ui'             => true,
				'show_in_menu'        => false,
				'show_in_admin_bar'   => false,
				'show_in_nav_menus'   => false,
				'can_export'          => true,
				'has_archive'         => false,
				'hierarchical'        => false,
				'exclude_from_search' => true,
				'show_in_rest'        => false,
				'publicly_queryable'  => false,
				'rewrite'             => array( 'slug' => 'wpadcenter-banners' ),
				'capability_type'     => 'post',
			)
		);
		return apply_filters( 'wpadcenter_cpt_args', $cpt_args );
	}

	/**
	 * Register CPT's.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_cpt() {
		$wpadcenter_cpt_args = $this->wpadcenter_get_cpt_args();
		foreach ( $wpadcenter_cpt_args as $key => $cpt_args ) {
			if ( ! post_type_exists( 'wpadcenter-' . $key ) ) {
				register_post_type( 'wpadcenter-' . $key, $cpt_args );
			}
		}
	}

	/**
	 * Register custom statuses for CPT - Campaigns.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_cpt_campaigns_status() {
		$campaigns_statuses = apply_filters(
			'wpadcenter_campaigns_statuses',
			array(
				'adc-paused'    => array(
					'label'                     => _x( 'Paused', 'Campaign Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of campaigns */
					'label_count'               => _n_noop(
						'Paused <span class="count">(%s)</span>',
						'Paused <span class="count">(%s)</span>',
						'wpadcenter'
					),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
				'adc-running'   => array(
					'label'                     => _x( 'Running', 'Campaign Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of campaigns */
					'label_count'               => _n_noop(
						'Running <span class="count">(%s)</span>',
						'Running <span class="count">(%s)</span>',
						'wpadcenter'
					),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
				'adc-scheduled' => array(
					'label'                     => _x( 'Scheduled', 'Campaign Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of campaigns */
					'label_count'               => _n_noop(
						'Scheduled <span class="count">(%s)</span>',
						'Scheduled <span class="count">(%s)</span>',
						'wpadcenter'
					),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
				'adc-finished'  => array(
					'label'                     => _x( 'Finished', 'Campaign Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of campaigns */
					'label_count'               => _n_noop(
						'Finished <span class="count">(%s)</span>',
						'Finished <span class="count">(%s)</span>',
						'wpadcenter'
					),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
			)
		);

		foreach ( $campaigns_statuses as $campaign_status => $values ) {
			register_post_status( $campaign_status, $values );
		}
	}

	/**
	 * Register block.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_block_type() {
		if ( ! function_exists( 'register_block_type' ) ) {
			return;
		}
		$ad_zones      = array();
		$zone['label'] = __( 'Select Ad Zone', 'wpadcenter' );
		$zone['value'] = '0';
		$ad_zones[]    = $zone;
		$args          = array(
			'post_type'   => 'wpadcenter-adzones',
			'post_status' => 'publish',
			'numberposts' => -1,
		);
		$adzones       = get_posts( $args );
		if ( $adzones ) {
			foreach ( $adzones as $adzone ) {
				$zone['value'] = $adzone->ID;
				$zone['label'] = $adzone->post_title;
				$ad_zones[]    = $zone;
			}
		}
		wp_register_style(
			$this->plugin_name . '-block',
			plugin_dir_url( __FILE__ ) . 'css/wpadcenter-admin-block' . WPADCENTER_SCRIPT_SUFFIX . '.css',
			array(),
			$this->version,
			'all'
		);
		wp_register_script(
			$this->plugin_name . '-cycle2',
			plugin_dir_url( __FILE__ ) . 'js/libraries/cycle2/cycle2.js',
			array(
				'jquery',
			),
			$this->version,
			true
		);
		wp_register_script(
			$this->plugin_name . '-block',
			plugin_dir_url( __FILE__ ) . 'js/blocks/wpadcenter-admin-block.js',
			array(
				'jquery',
				'wp-blocks',
				'wp-i18n',
				'wp-editor',
				'wp-element',
				'wp-components',
			),
			$this->version,
			false
		);
		wp_localize_script( $this->plugin_name . '-block', 'ad_zones', $ad_zones );
		register_block_type(
			'wpadcenter/block',
			array(
				'editor_script'   => $this->plugin_name . '-block',
				'render_callback' => array( $this, 'wpadcenter_block_render_callback' ),
				'attributes'      => array(
					'id' => array(
						'default' => '0',
						'type'    => 'string,',
					),
				),
			)
		);
	}

	/**
	 * Render callback for block.
	 *
	 * @since 1.0.0
	 *
	 * @param Array $atts Shortcode attributes.
	 * @return string
	 */
	public function wpadcenter_block_render_callback( $atts ) {

		if ( function_exists( 'get_current_screen' ) ) {
			$screen = get_current_screen();
			if ( method_exists( $screen, 'is_block_editor' ) ) {
				wp_enqueue_script( $this->plugin_name . '-cycle2' );
				wp_enqueue_style( $this->plugin_name . '-block' );
			}
		} elseif ( has_block( 'wpadcenter/block', get_the_ID() ) ) {
			wp_enqueue_script( $this->plugin_name . '-frontend' );
			wp_enqueue_script( $this->plugin_name . '-cycle2' );
			wp_enqueue_style( $this->plugin_name . '-block' );
			wp_enqueue_script( $this->plugin_name . '-block' );
			wp_localize_script(
				$this->plugin_name . '-frontend',
				'ajax_url',
				array(
					'url'      => admin_url( 'admin-ajax.php' ),
					'security' => wp_create_nonce( 'wpadcenter_set_clicks' ),
				)
			);
		}

		$container_styles = '';
		$message          = '';
		if ( isset( $atts['id'] ) && '0' !== $atts['id'] ) {
			$adzone_id    = sanitize_text_field( wp_unslash( $atts['id'] ) );
			$the_options  = Wpadcenter::wpadcenter_get_settings();
			$geo_location = $the_options['geo_location'];
			$auto_refresh = $the_options['auto_refresh'];
			if ( $auto_refresh ) {
				$transition = $the_options['transition_effect'];
				$speed      = $the_options['transition_speed'];
				$delay      = $the_options['transition_delay'];
			}
			$numberofposts     = -1;
			$container_styles .= 'width:100%;height:auto;margin:0;padding:0;';
			$adzone_display    = true;
			if ( 'adzone' === $geo_location && get_option( 'wpadcenter_pro_active' ) ) {
				$adzone_display = apply_filters( 'wpadcenter_geolocation_adzone', $adzone_id );
			}
			if ( $adzone_display ) {
				$auto_refresh_override = get_field( 'auto_refresh_ads', $adzone_id );
				if ( $auto_refresh_override ) {
					$auto_refresh = $auto_refresh_override;
					$transition   = get_field( 'transition_effect', $adzone_id );
					$speed        = get_field( 'transition_speed', $adzone_id );
					$delay        = get_field( 'transition_delay', $adzone_id );
				}
				$adzone_default_banner = get_field( 'default_banner', $adzone_id );
				$adzone_signup_link    = get_field( 'signup_link', $adzone_id );
				$adzone_template       = get_field( 'template', $adzone_id );
				if ( $adzone_template && '0' !== $adzone_template ) {
					$adzone_size = explode( 'x', $adzone_template );
				} else {
					$adzone_size[0] = get_field( 'custom_width', $adzone_id );
					$adzone_size[1] = get_field( 'custom_height', $adzone_id );
				}
				$adzone_text_above = get_field( 'text_above_ad_zone', $adzone_id );
				$adzone_text_below = get_field( 'text_below_ad_zone', $adzone_id );
				$width             = $adzone_size['0'];
				$height            = $adzone_size['1'];
				$adzone_rows       = get_field( 'horizontal', $adzone_id );
				$adzone_columns    = get_field( 'vertical', $adzone_id );
				$adzone_classes    = 'adzone_1_1';
				if ( $adzone_rows && $adzone_columns && ! $auto_refresh ) {
					$adzone_classes = 'adzone_' . $adzone_rows . '_' . $adzone_columns;
					$numberofposts  = $adzone_columns * $adzone_rows;
				}
				$args      = array(
					'post_type'   => 'wpadcenter-campaigns',
					'post_status' => 'adc-running',
					'numberposts' => -1,
					'orderby'     => 'rand',
					'order'       => 'ASC',
				);
				$campaigns = get_posts( $args );
				if ( $campaigns ) {
					$campaign_ids = array();
					if ( 'campaign' === $geo_location && get_option( 'wpadcenter_pro_active' ) ) {
						$campaign_ids = apply_filters( 'wpadcenter_geolocation_campaigns', $campaigns );
					} else {
						foreach ( $campaigns as $campaign ) {
							$campaign_ids[] = $campaign->ID;
						}
					}
					if ( empty( $campaign_ids ) ) {
						$campaign_ids = '';
					}
					$args    = array(
						'post_type'   => 'wpadcenter-banners',
						'post_status' => 'publish',
						'numberposts' => $numberofposts,
						'orderby'     => 'rand',
						'order'       => 'ASC',
						'meta_query'  => array( // phpcs:ignore slow query
							'relation' => 'AND',
							array(
								'key'     => 'banners_campaign',
								'value'   => $campaign_ids,
								'compare' => 'IN',
							),
							array(
								'key'     => 'banners_ad_zone',
								'value'   => $adzone_id,
								'compare' => '=',
							),
						),
					);
					$banners = get_posts( $args );
					if ( $banners ) {
						$message  = apply_filters( 'wpadcenter_block_render_before', $message );
						$message .= '<div class="wpadcenter-text-above">' . $adzone_text_above . '</div>';
						if ( $auto_refresh ) {
							$container_styles .= 'max-width:' . $width . 'px;';
							$message          .= '<div class="wpadcenter-slider cycle-slideshow" data-slides="div.wpadcenter-slide" data-cycle-fx="' . $transition . '" data-cycle-speed="' . $speed . '" data-cycle-delay="' . $delay . '" data-cycle-auto-height="container">';
						} else {
							$message .= '<div class="wpadcenter-slider">';
						}
						foreach ( $banners as $banner ) {
							$banner_id          = $banner->ID;
							$banner_link_url    = get_field( 'link_url', $banner_id );
							$banner_link_target = get_field( 'link_target', $banner_id );
							if ( $banner_link_target && '_blank' !== $banner_link_target && true !== $banner_link_target ) {
								$banner_link_target = '_self';
							} elseif ( $banner_link_target ) {
								$banner_link_target = '_blank';
							} else {
								$banner_link_target = '_self';
							}
							$banner_link_nofollow = get_field( 'link_nofollow', $banner_id );
							$banner_advertiser    = get_field( 'banners_advertiser', $banner_id );
							$banner_campaign      = get_field( 'banners_campaign', $banner_id );
							$banner_ad_zone       = get_field( 'banners_ad_zone', $banner_id );
							$banner_type          = get_field( 'banner_type', $banner_id );
							$message             .= '<div class="wpadcenter-slide ' . $adzone_classes . '">';
							if ( ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) && get_option( 'wpadcenter_pro_active' ) ) {
								$message .= '<input type="hidden" name="banner_id" value="' . $banner_id . '">';
								$message .= '<input type="hidden" name="banner_advertiser" value="' . $banner_advertiser . '">';
								$message .= '<input type="hidden" name="banner_campaign" value="' . $banner_campaign . '">';
								$message .= '<input type="hidden" name="banner_ad_zone" value="' . $banner_ad_zone . '">';
								if ( get_option( 'wpadcenter_pro_active' ) ) {
									Wpadcenter::wpadcenter_set_impressions(
										$banner_id,
										$banner_campaign,
										$banner_advertiser,
										$banner_ad_zone
									);
								}
							}
							if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
								$message .= '<a class="link" href="' . $banner_link_url . '" target="' . $banner_link_target . '"';
								if ( $banner_link_nofollow ) {
									$message .= ' rel="nofollow"';
								}
								$message .= '>';
							}
							switch ( $banner_type ) {
								case 'image':
									$banner_img = get_field( 'image', $banner_id );
									$banner_img = wp_get_attachment_image( $banner_img, $width, $height );
									$message   .= $banner_img;
									break;
								case 'external_image_link':
									$banner_img_link = get_field( 'image_link', $banner_id );
									$message        .= '<img width="' . $width . '" height="' . $height . '" src="' . esc_url(
										$banner_img_link
									) . '"/>';
									break;
								case 'ad_code':
									$banner_ad_code = get_field( 'ad_code', $banner_id );
									$message       .= '<span class="wpadcenter-ad-code">' . $banner_ad_code . '</span>';
									break;
								case 'adsense_code':
									$banner_ad_code = get_field( 'field_google_adsense_code', $banner_id );
									$message       .= '<div style="min-height:200px;min-width: 200px" class="wpadcenter-ad-code">' . $banner_ad_code . '</div>';
									break;
							}
							if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
								$message .= '</a>';
							}
							$message .= '</div>';
						}
						$message .= '</div>';
						$message .= '<div class="wpadcenter-text-below">' . $adzone_text_below . '</div>';
					} else {
						if ( $adzone_default_banner ) {
							if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
								$message .= '<a href="' . $adzone_signup_link . '" target="_blank">';
							}
							$default_banner_img = wp_get_attachment_image( $adzone_default_banner, $width, $height );
							$message           .= $default_banner_img;
							if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
								$message .= '</a>';
							}
						} elseif ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
							$message .= __( 'No Ads found.', 'wpadcenter' );
						}
					}
				} else {
					if ( $adzone_default_banner ) {
						if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
							$message .= '<a href="' . $adzone_signup_link . '" target="_blank">';
						}
						$default_banner_img = wp_get_attachment_image( $adzone_default_banner, $width, $height );
						$message           .= $default_banner_img;
						if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
							$message .= '</a>';
						}
					} elseif ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
						$message .= __( 'No Campaigns found.', 'wpadcenter' );
					}
				}
			} elseif ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
				$message .= __( 'No Ads found.', 'wpadcenter' );
			}
		} elseif ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
			$message .= __( 'Please choose ad zone.', 'wpadcenter' );
		}
		$message = apply_filters( 'wpadcenter_block_render_after', $message );
		return '<div class="wpadcenter-block-container" style="' . $container_styles . '">' . $message . '</div>';
	}

	/**
	 * Prints a combobox based on options and selected=match value.
	 *
	 * @since 1.0.0
	 * @param array  $options Array of options.
	 * @param string $selected Which of those options should be selected (allows just one; is case sensitive).
	 */
	public function print_combobox_options( $options, $selected ) {
		foreach ( $options as $key => $value ) {
			echo '<option value="' . esc_html( $value ) . '"';
			if ( $value === $selected ) {
				echo ' selected="selected"';
			}
			echo '>' . esc_html( $key ) . '</option>';
		}
	}

	/**
	 * Return transition effect options.
	 *
	 * @since 1.0.0
	 */
	public function get_transition_effect_options() {
		return apply_filters(
			'wpadcenter_transition_effect_options',
			array(
				__( 'None', 'wpadcenter' )                 => 'none',
				__( 'Fade', 'wpadcenter' )                 => 'fade',
				__( 'Fade Out', 'wpadcenter' )             => 'fadeout',
				__( 'Scroll Right to Left', 'wpadcenter' ) => 'scrollHorz',
				__( 'Scroll Left to Right', 'wpadcenter' ) => 'scrollHorzReverse',
				__( 'Scroll Top to Bottom', 'wpadcenter' ) => 'scrollVert',
				__( 'Scroll Bottom to Top', 'wpadcenter' ) => 'scrollVertReverse',
			)
		);
	}

	/**
	 * Define CPT's messages.
	 *
	 * @since 1.0.0
	 * @param array $messages Default Post Messages.
	 * @return mixed
	 */
	public function wpadcenter_cpt_update_messages( $messages ) {

		$wpadcenter_cpts = apply_filters(
			'wpadcenter_cpts',
			array(
				'wpadcenter-adzones'   => __( 'Ad Zone', 'wpadcenter' ),
				'wpadcenter-campaigns' => __( 'Campaign', 'wpadcenter' ),
				'wpadcenter-banners'   => __( 'Ad', 'wpadcenter' ),
			)
		);

		foreach ( $wpadcenter_cpts as $cpt => $val ) {
			$messages[ $cpt ] = array(
				0 => '', // Unused. Messages start at index 1.
				/* translators: %s: custom post type */
				1 => sprintf( __( '%s updated.', 'wpadcenter' ), $val ),
				/* translators: %s: custom post type */
				4 => sprintf( __( '%s updated.', 'wpadcenter' ), $val ),
				/* translators: %s: custom post type */
				6 => sprintf( __( '%s saved.', 'wpadcenter' ), $val ),
				/* translators: %s: custom post type */
				7 => sprintf( __( '%s saved.', 'wpadcenter' ), $val ),
				/* translators: %s: custom post type */
				8 => sprintf( __( '%s saved.', 'wpadcenter' ), $val ),
			);
		}

		return $messages;
	}

	/**
	 * Remove quick edit option for CPT's.
	 *
	 * @since 1.0.0
	 * @param array  $actions Post actions.
	 * @param Object $post Post.
	 * @return mixed
	 */
	public function wpadcenter_remove_cpt_quick_edit_action( $actions, $post ) {
		$wpadcenter_cpts = array( 'wpadcenter-adzones', 'wpadcenter-campaigns', 'wpadcenter-banners' );
		if ( in_array( $post->post_type, $wpadcenter_cpts, true ) ) {
			unset( $actions['inline hide-if-no-js'] );
		}
		$post_type_object = get_post_type_object( $post->post_type );
		$title            = _draft_or_post_title();
		if ( 'wpadcenter-campaigns' === $post->post_type ) {
			if ( 'adc-paused' === $post->post_status ) {
				$actions['adc-run'] = sprintf(
					/* translators: 1: href link 2: aria label 3: Text */
					'<a href="%s" aria-label="%s">%s</a>',
					wp_nonce_url(
						admin_url( sprintf( $post_type_object->_edit_link . '&amp;action=adc-run', $post->ID ) ),
						'adc-run-post_' . $post->ID
					),
					esc_attr(
						sprintf(
							/* translators: %s: post title */
							__( 'Re-Run &#8220;%s&#8221;', 'wpadcenter' ),
							$title
						)
					),
					__( 'Re-Run', 'wpadcenter' )
				);
			}
			if ( 'adc-running' === $post->post_status ) {
				$actions['adc-pause'] = sprintf(
					/* translators: 1: href link 2: aria label 3: Text */
					'<a href="%1s" aria-label="%2s">%3s</a>',
					wp_nonce_url(
						admin_url( sprintf( $post_type_object->_edit_link . '&amp;action=adc-pause', $post->ID ) ),
						'adc-pause-post_' . $post->ID
					),
					esc_attr(
						sprintf(
							/* translators: %s: post title */
							__( 'Pause &#8220;%s&#8221;', 'wpadcenter' ),
							$title
						)
					),
					__( 'Pause', 'wpadcenter' )
				);
			}
		}
		return $actions;
	}

	/**
	 * Remove bulk edit options for CPT's.
	 *
	 * @since 1.0.0
	 * @param array $actions Post actions.
	 * @return mixed
	 */
	public function wpadcenter_remove_cpt_bulk_edit_action( $actions ) {
		unset( $actions['edit'] );
		return $actions;
	}

	/**
	 * Hides publishing actions for CPT's.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_hide_cpt_publishing_actions() {
		global $post;
		$wpadcenter_cpts = array( 'wpadcenter-adzones', 'wpadcenter-campaigns', 'wpadcenter-banners' );
		if ( in_array( $post->post_type, $wpadcenter_cpts, true ) ) {
			echo '
                <style type="text/css">
                    #misc-publishing-actions,
                    #minor-publishing-actions,
                    #delete-action{
                        display:none;
                    }
                </style>
            ';
		}
	}

	/**
	 * Updated CPT's title text.
	 *
	 * @since 1.0.0
	 * @param String $title Post title.
	 * @return string
	 */
	public function wpadcenter_change_cpt_default_title( $title ) {
		global $current_screen;

		if ( 'wpadcenter-adzones' === $current_screen->post_type ) {
			$title = __( 'Ad Zone Name', 'wpadcenter' );
		} elseif ( 'wpadcenter-campaigns' === $current_screen->post_type ) {
			$title = __( 'Campaign Name', 'wpadcenter' );
		} elseif ( 'wpadcenter-banners' === $current_screen->post_type ) {
			$title = __( 'Ad Name', 'wpadcenter' );
		}

		return $title;
	}

	/**
	 * Updated CPT's publish button text.
	 *
	 * @since 1.0.0
	 * @param String $translation Post translated text.
	 * @param String $text Post publish button text.
	 * @return string
	 */
	public function wpadcenter_change_cpt_publish_button( $translation, $text ) {
		if ( 'wpadcenter-adzones' === get_post_type() ) {
			if ( 'Publish' === $text ) {
				return __( 'Save Ad Zone', 'wpadcenter' );
			} elseif ( 'Update' === $text ) {
				return __( 'Update Ad Zone', 'wpadcenter' );
			}
		} elseif ( 'wpadcenter-campaigns' === get_post_type() ) {
			if ( 'Publish' === $text ) {
				return __( 'Save Campaign', 'wpadcenter' );
			} elseif ( 'Update' === $text ) {
				return __( 'Update Campaign', 'wpadcenter' );
			}
		} elseif ( 'wpadcenter-banners' === get_post_type() ) {
			if ( 'Publish' === $text ) {
				return __( 'Save Ad', 'wpadcenter' );
			} elseif ( 'Update' === $text ) {
				return __( 'Update Ad', 'wpadcenter' );
			}
		}
		return $translation;
	}

	/**
	 * Manage Ad Zones columns
	 *
	 * @since 1.0.0
	 * @return array|void
	 */
	public function wpadcenter_manage_edit_adzones_columns() {
		global $current_screen;
		if ( 'wpadcenter-adzones' !== $current_screen->post_type ) {
			return;
		}
		$columns = array(
			'cb'               => '<input type="checkbox" />',
			'title'            => __( 'Name', 'wpadcenter' ),
			'size'             => __( 'Size', 'wpadcenter' ),
			'auto_refresh_ads' => __( 'Auto Refresh Ads', 'wpadcenter' ),
			'shortcode'        => __( 'Shortcode', 'wpadcenter' ),
			'date'             => __( 'Date', 'wpadcenter' ),
		);
		return $columns;
	}

	/**
	 * Manage Campaigns columns
	 *
	 * @since 1.0.0
	 * @return array|void
	 */
	public function wpadcenter_manage_edit_campaigns_columns() {
		global $current_screen;
		if ( 'wpadcenter-campaigns' !== $current_screen->post_type ) {
			return;
		}
		$columns = array(
			'cb'               => '<input type="checkbox" />',
			'title'            => __( 'Name', 'wpadcenter' ),
			'advertiser'       => __( 'Advertiser', 'wpadcenter' ),
			'start_date'       => __( 'Start Date', 'wpadcenter' ),
			'end_date'         => __( 'End Date', 'wpadcenter' ),
			'campaigns_status' => __( 'Status', 'wpadcenter' ),
		);
		return $columns;
	}

	/**
	 * Manage Ads columns
	 *
	 * @since 1.0.0
	 * @return array|void
	 */
	public function wpadcenter_manage_edit_banners_columns() {
		global $current_screen;
		if ( 'wpadcenter-banners' !== $current_screen->post_type ) {
			return;
		}
		$columns = array(
			'cb'          => '<input type="checkbox" />',
			'title'       => __( 'Name', 'wpadcenter' ),
			'banner_type' => __( 'Type', 'wpadcenter' ),
			'advertiser'  => __( 'Advertiser', 'wpadcenter' ),
			'campaign'    => __( 'Campaign', 'wpadcenter' ),
			'ad_zone'     => __( 'Ad Zone', 'wpadcenter' ),
			'date'        => __( 'Date', 'wpadcenter' ),
		);
		return $columns;
	}

	/**
	 * Render Ad Zones columns.
	 *
	 * @since 1.0.0
	 * @param String $column Column.
	 */
	public function wpadcenter_manage_custom_adzones_columns( $column ) {
		global $post;
		global $current_screen;

		if ( 'wpadcenter-adzones' !== $current_screen->post_type ) {
			return;
		}

		switch ( $column ) {
			case 'size':
				$template = get_field( 'template', $post->ID );
				if ( $template && '0' !== $template ) {
					echo esc_attr( get_field( 'template', $post->ID ) );
				} else {
					echo esc_attr( get_field( 'custom_width', $post->ID ) ) . 'x' . esc_attr(
						get_field( 'custom_height', $post->ID )
					);
				}
				break;
			case 'auto_refresh_ads':
				$auto_refresh_ads = get_field( 'auto_refresh_ads', $post->ID );
				if ( isset( $auto_refresh_ads ) && true === $auto_refresh_ads ) {
					esc_attr_e( 'Yes', 'wpadcenter' );
				} else {
					esc_attr_e( 'No', 'wpadcenter' );
				}
				break;
			case 'shortcode':
				echo '[wpadcenter id=' . esc_attr( $post->ID ) . ']';
				break;
		}
	}

	/**
	 * Render Campaigns columns.
	 *
	 * @since 1.0.0
	 * @param String $column Column.
	 */
	public function wpadcenter_manage_custom_campaigns_columns( $column ) {
		global $post;
		global $current_screen;

		if ( 'wpadcenter-campaigns' !== $current_screen->post_type ) {
			return;
		}

		switch ( $column ) {
			case 'advertiser':
				$advertiser = get_field( 'campaigns_advertiser', $post->ID );
				$advert     = get_userdata( $advertiser );
				if ( $advert ) {
					echo '<a class="row-title" href="' . esc_url(
						get_edit_user_link( $advertiser )
					) . '" aria-label="' . esc_attr( $advert->display_name ) . ' (Edit)">' . esc_attr(
						$advert->display_name
					) . '</a>';
				} else {
					echo esc_attr__( 'Advertiser deleted', 'wpadcenter' );
				}
				break;
			case 'start_date':
				echo esc_attr( get_field( 'start_date', $post->ID ) );
				break;
			case 'end_date':
				echo esc_attr( get_field( 'end_date', $post->ID ) );
				break;
			case 'campaigns_status':
				echo esc_attr( ucfirst( get_field( 'campaigns_status', $post->ID ) ) );
				break;
		}
	}

	/**
	 * Render Ads columns.
	 *
	 * @since 1.0.0
	 * @param String $column Column.
	 */
	public function wpadcenter_manage_custom_banners_columns( $column ) {
		global $post;
		global $current_screen;

		if ( 'wpadcenter-banners' !== $current_screen->post_type ) {
			return;
		}

		switch ( $column ) {
			case 'banner_type':
				echo esc_attr( ucwords( str_replace( '_', ' ', get_field( 'banner_type', $post->ID ) ) ) );
				break;
			case 'advertiser':
				$advertiser = get_field( 'banners_advertiser', $post->ID );
				$advert     = get_userdata( $advertiser );
				if ( $advert ) {
					echo '<a class="row-title" href="' . esc_url(
						get_edit_user_link( $advertiser )
					) . '" aria-label="' . esc_attr( $advert->display_name ) . ' (Edit)">' . esc_attr(
						$advert->display_name
					) . '</a>';
				} else {
					echo esc_attr__( 'Advertiser deleted', 'wpadcenter' );
				}
				break;
			case 'campaign':
				$campaign = get_field( 'banners_campaign', $post->ID );
				if ( get_post( $campaign ) ) {
					echo '<a class="row-title" href="' . esc_url(
						get_edit_post_link( $campaign )
					) . '" aria-label="' . esc_attr( get_the_title( $campaign ) ) . ' (Edit)">' . esc_attr(
						get_the_title( $campaign )
					) . '</a>';
				} else {
					echo esc_attr__( 'Campaign deleted', 'wpadcenter' );
				}
				break;
			case 'ad_zone':
				$ad_zone = get_field( 'banners_ad_zone', $post->ID );
				if ( get_post( $ad_zone ) ) {
					echo '<a class="row-title" href="' . esc_url(
						get_edit_post_link( $ad_zone )
					) . '" aria-label="' . esc_attr( get_the_title( $ad_zone ) ) . ' (Edit)">' . esc_attr(
						get_the_title( $ad_zone )
					) . '</a>';
				} else {
					echo esc_attr__( 'Ad Zone deleted', 'wpadcenter' );
				}
				break;
		}
	}

	/**
	 * Save Campaigns CPT.
	 *
	 * @since 1.0.0
	 * @param int $post_id Post ID.
	 */
	public function wpadcenter_campaigns_save_custom_post( $post_id ) {
		$post = get_post( $post_id );
		if ( 'wpadcenter-campaigns' !== $post->post_type ) {
			return;
		}
		if ( $post_id ) {
			$campaigns_status = get_field( 'campaigns_status', $post_id );
			if ( $campaigns_status ) {
				$today      = gmdate( 'd-m-Y' );
				$start_date = str_replace( '/', '-', get_field( 'start_date', $post_id ) );
				$end_date   = str_replace( '/', '-', get_field( 'end_date', $post_id ) );
				if ( 'paused' !== $campaigns_status ) {
					$campaigns_status = 'running';
					if ( ! empty( $start_date ) && strtotime( $start_date ) > strtotime( $today ) ) {
						$campaigns_status = 'scheduled';
					}
					if ( ! empty( $end_date ) && strtotime( $end_date ) < strtotime( $today ) ) {
						$campaigns_status = 'finished';
					}
					update_field( 'campaigns_status', $campaigns_status, $post_id );
				}
				if ( 'trash' !== $post->post_status ) {
					remove_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );
					wp_update_post(
						array(
							'ID'          => $post_id,
							'post_status' => 'adc-' . trim( $campaigns_status ),
						)
					);
					add_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );
				}
			}
		}
	}


	/**
	 * Post row action - Re-run campaign.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_re_run_campaigns() {
		$sendback = wp_get_referer();
		$sendback = remove_query_arg( array( 'campaign_running' ), $sendback );

		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
		} else {
			$post_id = 0;
		}
		if ( $post_id ) {
			$post = get_post( $post_id );
		}
		if ( $post ) {
			$post_type        = $post->post_type;
			$post_type_object = get_post_type_object( $post_type );
		}
		check_admin_referer( 'adc-run-post_' . $post_id );
		if ( ! $post ) {
			wp_die( esc_attr__( 'The campaign you are trying to re-run no longer exists.', 'wpadcenter' ) );
		}

		if ( ! $post_type_object ) {
			wp_die( esc_attr__( 'Invalid post type.', 'wpadcenter' ) );
		}

		update_field( 'campaigns_status', 'running', $post_id );
		$post->post_status = 'adc-running';

		$result = wp_update_post( $post );

		if ( $result ) {
			$sendback = add_query_arg( 'campaign_running', true, $sendback );
		} else {
			$sendback = add_query_arg( 'campaign_running', false, $sendback );
		}
		wp_safe_redirect( $sendback ? $sendback : admin_url( 'edit.php?post_type=wpadcenter-campaigns' ) );
		exit;
	}

	/**
	 * Post row action - Pause campaign.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_pause_campaigns() {
		$sendback = wp_get_referer();
		$sendback = remove_query_arg( array( 'campaign_paused' ), $sendback );

		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
		} else {
			$post_id = 0;
		}
		if ( $post_id ) {
			$post = get_post( $post_id );
		}
		if ( $post ) {
			$post_type        = $post->post_type;
			$post_type_object = get_post_type_object( $post_type );
		}
		check_admin_referer( 'adc-pause-post_' . $post_id );
		if ( ! $post ) {
			wp_die( esc_attr__( 'The campaign you are trying to pause no longer exists.', 'wpadcenter' ) );
		}

		if ( ! $post_type_object ) {
			wp_die( esc_attr__( 'Invalid post type.', 'wpadcenter' ) );
		}

		update_field( 'campaigns_status', 'paused', $post_id );
		$post->post_status = 'adc-paused';

		remove_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );
		$result = wp_update_post( $post );
		add_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );

		if ( $result ) {
			$sendback = add_query_arg( 'campaign_paused', true, $sendback );
		} else {
			$sendback = add_query_arg( 'campaign_paused', false, $sendback );
		}
		wp_safe_redirect( $sendback ? $sendback : admin_url( 'edit.php?post_type=wpadcenter-campaigns' ) );
		exit;
	}

	/**
	 * Return custom status for Adverts CPT.
	 *
	 * @since 1.0.0
	 * @param string $key Status.
	 * @return mixed
	 */
	public function get_adverts_status( $key = '' ) {
		$adverts_status = apply_filters(
			'wpadcenter_adverts_status',
			array(
				'adc-approved' => __( 'Approved', 'wpadcenter' ),
				'adc-pending'  => __( 'Pending', 'wpadcenter' ),
				'adc-declined' => __( 'Declined', 'wpadcenter' ),
			)
		);
		return '' !== $key ? $adverts_status[ $key ] : $adverts_status;
	}

	/**
	 * Return custom status for Campaigns CPT.
	 *
	 * @since 1.0.0
	 * @param string $key Status.
	 * @return mixed
	 */
	public function get_campaigns_status( $key = '' ) {
		$campaigns_status = apply_filters(
			'wpadcenter_campaigns_status',
			array(
				'adc-paused'    => __( 'Paused', 'wpadcenter' ),
				'adc-running'   => __( 'Running', 'wpadcenter' ),
				'adc-scheduled' => __( 'Scheduled', 'wpadcenter' ),
				'adc-finished'  => __( 'Finished', 'wpadcenter' ),
			)
		);
		return '' !== $key ? $campaigns_status[ $key ] : $campaigns_status;
	}

	/**
	 * Convert CPT's custom post status.
	 *
	 * @since 1.0.0
	 * @param String $post_status Post Status.
	 * @param Object $post Post object.
	 * @return mixed
	 */
	public function wpadcenter_acf_get_post_status( $post_status, $post ) {
		if ( 'auto-draft' !== $post_status && 'trash' !== $post_status ) {
			if ( isset( $post->post_type ) ) {
				switch ( $post->post_type ) {
					case 'wpadcenter-campaigns':
						$post_status = $this->get_campaigns_status( $post_status );
						break;
				}
			}
		}
		return $post_status;
	}

	/**
	 * Filter acf user object result.
	 *
	 * @param string $result Result.
	 * @param object $user User.
	 * @param string $field Field.
	 * @param int    $post_id Post ID.
	 * @return mixed
	 */
	public function wpadcenter_filter_users_result( $result, $user, $field, $post_id ) {
		return $user->display_name;
	}


	/**
	 * Filter Campaign field for Ads CPT.
	 *
	 * @since 1.0.0
	 * @param Array $args Filter arguments.
	 * @param Array $field Field.
	 * @param int   $post_id Post ID.
	 * @return mixed
	 */
	public function wpadcenter_acf_banners_campaign_filter_post_query( $args, $field, $post_id ) {
		if ( isset( $_POST['advertiser'] ) ) { // phpcs:ignore CSRF ok
			$advertiser         = sanitize_text_field( wp_unslash( $_POST['advertiser'] ) ); // phpcs:ignore CSRF ok
			$args['meta_key']   = 'campaigns_advertiser'; // phpcs:ignore slow query
			$args['meta_value'] = $advertiser; // phpcs:ignore slow query
		}
		return $args;
	}

	/**
	 * Returns ACF Settings URL.
	 *
	 * @since 1.0.0
	 * @param  string $url Settings URL.
	 * @return string $url.
	 */
	public function wpadcenter_acf_settings_url( $url ) {
		if ( ! get_option( 'wpadcenter_pro_active' ) ) {
			$url = WPADCENTER_PLUGIN_ACF_URL;
		}
		return $url;
	}

	/**
	 * Show Admin Menu for ACF.
	 *
	 * @since 1.0.0
	 * @param  bool $show_admin Show Admin flag.
	 * @return bool $show_admin
	 */
	public function wpadcenter_acf_settings_show_admin( $show_admin ) {
		if ( ! get_option( 'wpadcenter_pro_active' ) ) {
			$show_admin = false;
		}
		return $show_admin;
	}

	/**
	 * Return false for wp meta box settings field value.
	 * Fix for conflict with Custom post type UI plugin.
	 *
	 * @param string $value Field value.
	 * @return bool
	 */
	public function wpadcenter_remove_wp_meta_box( $value ) {
		return false;
	}
}
