<?php
/**
 * The advertiser-specific functionality of the plugin.
 *
 * @link       https://club.wpeka.com/
 * @since      1.0.1
 *
 * @package    Wpadcenter
 * @subpackage Wpadcenter/includes
 */

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}



/**
 * The advertiser-specific functionality of the plugin.
 *
 * @package    Wpadcenter
 * @subpackage Wpadcenter/includes
 * @author     WPEka Club <support@wpeka.com>
 */
class Wpadcenter_Advertiser extends WP_List_Table {

	/**
	 * Wpadcenter_Advertiser constructor.
	 *
	 * @since 1.0.1
	 * @param array $args arguments.
	 */
	public function __construct( $args = array() ) {
		$args['singular'] = __( 'Advertiser', 'wpadcenter' );
		$args['plural']   = __( 'Advertisers', 'wpadcenter' );
		$args['ajax']     = false;
		parent::__construct( $args );
	}

	/**
	 * Retrieve advertisers.
	 *
	 * @since 1.0.1
	 * @param int $per_page Limit per page.
	 * @param int $page_number Current page number.
	 * @return array
	 */
	public static function get_advertisers( $per_page = 10, $page_number = 1 ) {
		$orderby     = isset( $_REQUEST['orderby'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['orderby'] ) ) : 'display_name'; // phpcs:ignore CSRF ok
		$order       = isset( $_REQUEST['order'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['order'] ) ) : 'ASC'; // phpcs:ignore CSRF ok
		$search      = isset( $_REQUEST['s'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['s'] ) ) : ''; // phpcs:ignore CSRF ok
		$users       = get_users(
			array(
				'role'           => 'advertiser',
				'number'         => $per_page,
				'offset'         => ( $page_number - 1 ) * $per_page,
				'orderby'        => $orderby,
				'order'          => $order,
				'search'         => '*' . $search . '*',
				'search_columns' => array( 'display_name', 'user_login', 'user_email' ),
			)
		);
		$advertisers = array();
		if ( $users ) {
			foreach ( $users as $user ) {
				$advertiser                    = array();
				$advertiser['ID']              = $user->ID;
				$advertiser['user_login']      = $user->user_login;
				$advertiser['user_email']      = $user->user_email;
				$advertiser['user_registered'] = $user->user_registered;
				$advertiser['display_name']    = $user->display_name;
				$advertisers[]                 = $advertiser;
			}
		}
		return $advertisers;
	}

	/**
	 * Delete advertiser.
	 *
	 * @since 1.0.1
	 * @param int $id Advertiser id.
	 */
	public static function delete_advertiser( $id ) {
		$args      = array(
			'post_type'   => 'wpadcenter-campaigns',
			'post_status' => array( 'adc-running', 'adc-scheduled' ),
			'numberposts' => -1,
			'meta_query'  => array( // phpcs:ignore slow query
				array(
					'key'     => 'campaigns_advertiser',
					'value'   => $id,
					'compare' => '=',
				),
			),
		);
		$campaigns = get_posts( $args );
		if ( $campaigns ) {
			foreach ( $campaigns as $campaign ) {
				update_field( 'campaigns_status', 'paused', $campaign->ID );
				wp_update_post(
					array(
						'ID'          => $campaign->ID,
						'post_status' => 'adc-paused',
					)
				);
			}
		}
		wp_delete_user( $id );
	}

	/**
	 * Return the count of advertisers.
	 *
	 * @since 1.0.1
	 * @return mixed
	 */
	public static function record_count() {
		$users_count = count_users();
		$count       = 0;
		if ( $users_count ) {
			$count = isset( $users_count['avail_roles']['advertiser'] ) ? $users_count['avail_roles']['advertiser'] : 0;
		}
		return $count;
	}

	/**
	 * Text displayed when no advertiser data is found.
	 *
	 * @since 1.0.1
	 */
	public function no_items() {
		esc_attr_e( 'No Advertisers found.', 'wpadcenter' );
	}

	/**
	 * Method for main column - display_name.
	 *
	 * @since 1.0.1
	 * @param array $item Item.
	 * @return string
	 */
	public function column_name( $item ) {
		$delete_nonce = wp_create_nonce( 'wpadcenter_delete_advertiser' );
		$title        = '<strong>' . $item['display_name'] . '</strong>';
		$page         = isset( $_REQUEST['page'] ) ? $_REQUEST['page'] : ''; // phpcs:ignore input var ok, CSRF ok, sanitization ok.
		$actions      = array(
			'edit'   => sprintf( '<a href="%s">Edit</a>', esc_url( get_edit_user_link( absint( $item['ID'] ) ) ) ),
			'delete' => sprintf( '<a href="?page=%s&action=%s&advertiser=%s&_wpnonce=%s">Delete</a>', esc_attr( $page ), 'delete', absint( $item['ID'] ), $delete_nonce ),
		);
		return $title . $this->row_actions( $actions );
	}

	/**
	 * Render a column when no column specific method exists.
	 *
	 * @since 1.0.1
	 *
	 * @param object $item Item.
	 * @param string $column_name Column name.
	 * @return string|void
	 */
	public function column_default( $item, $column_name ) {
		switch ( $column_name ) {
			case 'display_name':
				return $this->column_name( $item );
			case 'user_login':
			case 'user_email':
			case 'user_registered':
				return $item[ $column_name ];
		}
	}

	/**
	 * Render the bulk edit checkbox.
	 *
	 * @since 1.0.1
	 * @param object $item Item.
	 * @return string|void
	 */
	public function column_cb( $item ) {
		return sprintf(
			'<input type="checkbox" name="bulk-delete[]" value="%s" />',
			$item['ID']
		);
	}

	/**
	 * Associative array of columns.
	 *
	 * @return array
	 */
	public function get_columns() {
		$columns = array(
			'cb'              => '<input type="checkbox" />',
			'display_name'    => __( 'Name', 'wpadcenter' ),
			'user_login'      => __( 'Username', 'wpadcenter' ),
			'user_email'      => __( 'Email', 'wpadcenter' ),
			'user_registered' => __( 'Created', 'wpadcenter' ),
		);
		return $columns;
	}

	/**
	 * Columns to make sortable.
	 *
	 * @return array
	 */
	public function get_sortable_columns() {
		$sortable_columns = array(
			'display_name' => array( 'display_name', true ),
		);
		return $sortable_columns;
	}

	/**
	 * Returns an associative array containing the bulk action.
	 *
	 * @return array
	 */
	public function get_bulk_actions() {
		$actions = array(
			'bulk-delete' => 'Delete',
		);

		return $actions;
	}

	/**
	 * Handles data query and filter, sorting, and pagination.
	 */
	public function prepare_items() {
		$this->_column_headers = $this->get_column_info();

		/** Process bulk action */
		$this->process_bulk_action();

		$per_page     = $this->get_items_per_page( 'advertisers_per_page', 10 );
		$current_page = $this->get_pagenum();
		$total_items  = self::record_count();

		$this->set_pagination_args(
			array(
				'total_items' => $total_items, // We have to calculate the total number of items.
				'per_page'    => $per_page, // We have to determine how many items to show on a page.
			)
		);
		$this->items = self::get_advertisers( $per_page, $current_page );
	}

	/**
	 * Process bulk actions.
	 */
	public function process_bulk_action() {

		// Detect when a bulk action is being triggered...
		if ( 'delete' === $this->current_action() ) {

			// In our file that handles the request, verify the nonce.
			$nonce = isset( $_REQUEST['_wpnonce'] ) ? esc_attr( $_REQUEST['_wpnonce'] ) : ''; // phpcs:ignore input var ok, CSRF ok, sanitization ok.

			if ( ! wp_verify_nonce( $nonce, 'wpadcenter_delete_advertiser' ) ) {
				die( 'Go get a life script kiddies' );
			} else {
				$advert = isset( $_GET['advertiser'] ) ? $_GET['advertiser'] : ''; // phpcs:ignore input var ok, CSRF ok, sanitization ok.
				if ( ! empty( $advert ) ) {
					self::delete_advertiser( absint( $advert ) );
				}
				wp_safe_redirect( esc_url( add_query_arg( array() ) ) );
				exit;
			}
		}

		// If the delete bulk action is triggered.
		if ( ( isset( $_POST['action'] ) && 'bulk-delete' === $_POST['action'] ) || ( isset( $_POST['action2'] ) && 'bulk-delete' === $_POST['action2'] ) ) { // phpcs:ignore input var ok, CSRF ok, sanitization ok.

			$delete_ids = isset( $_POST['bulk-delete'] ) ? esc_sql( $_POST['bulk-delete'] ) : array(); // phpcs:ignore input var ok, CSRF ok, sanitization ok.

			if ( ! empty( $delete_ids ) ) {
				// loop over the array of record IDs and delete them.
				foreach ( $delete_ids as $id ) {
					self::delete_advertiser( $id );
				}
			}
			wp_safe_redirect( esc_url( add_query_arg( array() ) ) );
			exit;
		}
	}
}
