<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://club.wpeka.com/
 * @since      1.0.0
 *
 * @package    Wpadcenter
 * @subpackage Wpadcenter/includes
 */

use Wpeka\Adcenter\Wpadcenter_Meta_Boxes;

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wpadcenter
 * @subpackage Wpadcenter/includes
 * @author     WPEka Club <support@wpeka.com>
 */
class Wpadcenter {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Wpadcenter_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * The currently stored option settings of the plugin.
	 *
	 * @since 1.0.0
	 * @access private
	 * @var array $stored_options The stored option settings of the plugin.
	 */
	private static $stored_options = array();

	const TOP = '# WPAdCenter ads.txt';

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'WPADCENTER_VERSION' ) ) {
			$this->version = WPADCENTER_VERSION;
		} else {
			$this->version = '1.1.0';
		}
		$this->plugin_name = 'wpadcenter';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wpadcenter_Loader. Orchestrates the hooks of the plugin.
	 * - Wpadcenter_I18n. Defines internationalization functionality.
	 * - Wpadcenter_Admin. Defines all hooks for the admin area.
	 * - Wpadcenter_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wpadcenter-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wpadcenter-public.php';

		/**
		 * The class responsible for defining all actions that occur in the widget area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-widget.php';

		/**
		 * The class responsible for defining all actions that occur for the advertiser.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-advertiser.php';

		/**
		 * The class responsible for defining all actions that occur in the acf.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/libraries/acf/acf.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-google-api.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-adsense.php';

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-meta-boxes.php';

		$this->loader = new Wpadcenter_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wpadcenter_I18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wpadcenter_I18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * What type of request is this?
	 *
	 * @since 1.0.1
	 * @param  string $type admin, ajax, cron or frontend.
	 * @return bool
	 */
	public static function is_request( $type ) {
		switch ( $type ) {
			case 'admin':
				return is_admin();
			case 'ajax':
				return defined( 'DOING_AJAX' );
			case 'cron':
				return defined( 'DOING_CRON' );
			case 'frontend':
				return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' ) && ! defined( 'REST_REQUEST' );
		}
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Wpadcenter_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		if ( self::is_request( 'admin' ) ) {
			$this->loader->add_action( 'admin_menu', $plugin_admin, 'wpadcenter_admin_menu' );
			$this->loader->add_action( 'admin_init', $plugin_admin, 'wpadcenter_admin_init' );
			$this->loader->add_action( 'acf/input/admin_enqueue_scripts', $plugin_admin, 'wpadcenter_enqueue_acf_scripts' );
			$this->loader->add_action( 'manage_edit-wpadcenter-adzones_columns', $plugin_admin, 'wpadcenter_manage_edit_adzones_columns' );
			$this->loader->add_action( 'manage_edit-wpadcenter-campaigns_columns', $plugin_admin, 'wpadcenter_manage_edit_campaigns_columns' );
			$this->loader->add_action( 'manage_edit-wpadcenter-banners_columns', $plugin_admin, 'wpadcenter_manage_edit_banners_columns' );

			$this->loader->add_action( 'manage_posts_custom_column', $plugin_admin, 'wpadcenter_manage_custom_adzones_columns' );
			$this->loader->add_action( 'manage_posts_custom_column', $plugin_admin, 'wpadcenter_manage_custom_campaigns_columns' );
			$this->loader->add_action( 'manage_posts_custom_column', $plugin_admin, 'wpadcenter_manage_custom_banners_columns' );

			$this->loader->add_action( 'save_post', $plugin_admin, 'wpadcenter_campaigns_save_custom_post' );

			$this->loader->add_action( 'post_action_adc-pause', $plugin_admin, 'wpadcenter_pause_campaigns' );

			$this->loader->add_action( 'admin_head-post.php', $plugin_admin, 'wpadcenter_hide_cpt_publishing_actions' );
			$this->loader->add_action( 'admin_head-post-new.php', $plugin_admin, 'wpadcenter_hide_cpt_publishing_actions' );
			$this->loader->add_action( 'post_updated_messages', $plugin_admin, 'wpadcenter_cpt_update_messages' );

			$this->loader->add_action( 'load-wpadcenter_page_wpadcenter-advertisers', $plugin_admin, 'wpadcenter_advertisers_screen_option' );
			$this->loader->add_filter( 'enter_title_here', $plugin_admin, 'wpadcenter_change_cpt_default_title' );
			$this->loader->add_filter( 'gettext', $plugin_admin, 'wpadcenter_change_cpt_publish_button', 10, 2 );
			$this->loader->add_filter( 'post_row_actions', $plugin_admin, 'wpadcenter_remove_cpt_quick_edit_action', 10, 2 );
			$this->loader->add_filter( 'bulk_actions-edit-wpadcenter-adzones', $plugin_admin, 'wpadcenter_remove_cpt_bulk_edit_action', 10, 1 );
			$this->loader->add_filter( 'bulk_actions-edit-wpadcenter-campaigns', $plugin_admin, 'wpadcenter_remove_cpt_bulk_edit_action', 10, 1 );
			$this->loader->add_filter( 'bulk_actions-edit-wpadcenter-banners', $plugin_admin, 'wpadcenter_remove_cpt_bulk_edit_action', 10, 1 );

			$this->loader->add_filter( 'get_post_status', $plugin_admin, 'wpadcenter_acf_get_post_status', 10, 2 );
			$this->loader->add_filter( 'acf/settings/url', $plugin_admin, 'wpadcenter_acf_settings_url', 10, 1 );
			$this->loader->add_filter( 'acf/settings/show_admin', $plugin_admin, 'wpadcenter_acf_settings_show_admin', 10, 1 );
			$this->loader->add_filter( 'acf/fields/post_object/query/key=field_banners_campaign', $plugin_admin, 'wpadcenter_acf_banners_campaign_filter_post_query', 10, 3 );
			$this->loader->add_filter( 'acf/fields/user/result', $plugin_admin, 'wpadcenter_filter_users_result', 10, 4 );
			$this->loader->add_filter( 'plugin_action_links_' . WPADCENTER_PLUGIN_BASENAME, $plugin_admin, 'wpadcenter_plugin_action_links' );
			$this->loader->add_filter( 'set-screen-option', $plugin_admin, 'wpadcenter_set_screen_option', 10, 3 );
			$this->loader->add_action( 'wp_ajax_get_wizard_data', $plugin_admin, 'wpadcenter_get_wizard_data' );
			$this->loader->add_action( 'wp_ajax_nopriv_get_wizard_data', $plugin_admin, 'wpadcenter_get_wizard_data' );
			$this->loader->add_action( 'wp_ajax_save_wizard_data', $plugin_admin, 'wpadcenter_save_wizard_data' );
			$this->loader->add_action( 'wp_ajax_nopriv_save_wizard_data', $plugin_admin, 'wpadcenter_save_wizard_data' );
			$this->loader->add_action( 'wp_ajax_check_ads_txt_problems', $plugin_admin, 'wpadcenter_check_ads_txt_problems' );
			$this->loader->add_action( 'wp_ajax_check_ads_txt_replace', $plugin_admin, 'wpadcenter_check_ads_txt_replace' );
			$this->loader->add_filter( 'wpadcenter_after_save_settings', $plugin_admin, 'wpadcenter_after_save_settings' );

		}

		$this->loader->add_action( 'init', $plugin_admin, 'wpadcenter_init' );
		$this->loader->add_action( 'init', $plugin_admin, 'wpadcenter_register_cpt' );
		$this->loader->add_action( 'init', $plugin_admin, 'wpadcenter_register_cpt_campaigns_status' );
		$this->loader->add_action( 'init', $plugin_admin, 'wpadcenter_register_block_type' );
		$this->loader->add_action( 'widgets_init', $plugin_admin, 'wpadcenter_register_widget' );
		$this->loader->add_action( 'wpadcenter_daily_cron', $plugin_admin, 'wpadcenter_daily_schedule_campaigns' );
		$this->loader->add_action( 'acf/init', Wpadcenter_Meta_Boxes::class, 'add_acf_fields' );
		$this->loader->add_filter( 'acf/settings/remove_wp_meta_box', $plugin_admin, 'wpadcenter_remove_wp_meta_box' );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Wpadcenter_Public( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		if ( self::is_request( 'frontend' ) ) {
			$this->loader->add_action( 'init', $plugin_public, 'wpadcenter_init' );
			$this->loader->add_action( 'template_redirect', $plugin_public, 'wpadcenter_template_redirect' );
			$this->loader->add_action( 'wp_ajax_set_clicks', $plugin_public, 'wpadcenter_set_clicks' );
			$this->loader->add_action( 'wp_ajax_set_clicks', $plugin_public, 'wpadcenter_set_clicks' );
		}

	}

	/**
	 * Returns default settings.
	 * If you override the settings here, be ultra careful to use escape characters.
	 *
	 * @since 1.0.0
	 * @param string $key Return default settings for particular key.
	 * @return mixed|void
	 */
	public static function wpadcenter_get_default_settings( $key = '' ) {
		$settings = array(
			// General settings.
			'notification'       => false,

			'auto_refresh'       => false,
			'transition_effect'  => 'none',
			'transition_speed'   => '500',
			'transition_delay'   => '1000',

			'enable_ads_txt'     => false,
			'ads_txt_content'    => '',
			'enable_scripts'     => false,
			'header_scripts'     => '',
			'body_scripts'       => '',
			'footer_scripts'     => '',

			'enable_advertisers' => false,

			'geo_location'       => 'none',
			'trim_stats'         => '0',

		);
		$settings = apply_filters( 'wpadcenter_default_settings', $settings );
		return '' !== $key ? $settings[ $key ] : $settings;
	}

	/**
	 * Returns sanitised content based on field-specific rules defined here
	 * used for both read AND write operations.
	 *
	 * @since 1.0.0
	 * @param string $key Key for the setting.
	 * @param string $value Value for the setting.
	 *
	 * @return bool|null|string
	 */
	public static function wpadcenter_sanitise_settings( $key, $value ) {
		$ret = null;
		switch ( $key ) {
			// Convert all boolean values from text to bool.
			case 'notification':
			case 'auto_refresh':
			case 'enable_scripts':
			case 'enable_advertisers':
			case 'enable_ads_txt':
				if ( 'true' === $value || true === $value ) {
					$ret = true;
				} elseif ( 'false' === $value || false === $value ) {
					$ret = false;
				} else {
					// Unexpected value returned from radio button, go fix the HTML.
					// Failover = assign null.
					$ret = 'fffffff';
				}
				break;
			case 'header_scripts':
			case 'body_scripts':
			case 'footer_scripts':
				$ret = trim( stripslashes( $value ) );
				break;
			case 'ads_txt_content':
				$ret = esc_textarea( $value );
				break;
			default:
				$ret = sanitize_text_field( $value );
				break;
		}
		if ( 'fffffff' === $ret ) {
			$ret = false;
		}
		return $ret;
	}

	/**
	 * Get current settings.
	 *
	 * @since 1.0.0
	 *
	 * @return array|mixed
	 */
	public static function wpadcenter_get_settings() {
		$settings             = self::wpadcenter_get_default_settings();
		self::$stored_options = get_option( WPADCENTER_SETTINGS_FIELD );
		if ( ! empty( self::$stored_options ) ) {
			foreach ( self::$stored_options as $key => $option ) {
				$settings[ $key ] = self::wpadcenter_sanitise_settings( $key, $option );
			}
		}
		update_option( WPADCENTER_SETTINGS_FIELD, $settings );
		return $settings;
	}

	/**
	 * Generate tab head for settings page,
	 * method will translate the string to current language.
	 *
	 * @since 1.0.0
	 *
	 * @param array $title_arr Tab labels.
	 */
	public static function wpadcenter_generate_settings_tabhead( $title_arr ) {
		foreach ( $title_arr as $k => $v ) {
			if ( is_array( $v ) ) {
				$v = ( isset( $v[2] ) ? $v[2] : '' ) . esc_attr( $v[0] ) . ' ' . ( isset( $v[1] ) ? $v[1] : '' );
			} else {
				$v = esc_attr( $v );
			}
			?>
			<a class="nav-tab" href="#<?php echo esc_html( $k ); ?>"><?php echo esc_html( $v ); ?></a>
			<?php
		}
	}

	/**
	 * Envelope settings tab content with tab div.
	 *
	 * @since 1.0.0
	 * @param string $view_file View file.
	 * @param string $target_id Target tab id.
	 */
	public static function wpadcenter_envelope_settings_tab( $view_file = '', $target_id ) {
		$the_options = self::wpadcenter_get_settings();
		?>
		<div class="wpadcenter-tab-content" data-id="<?php echo esc_attr( $target_id ); ?>">
			<?php
			if ( '' !== $view_file && file_exists( $view_file ) ) {
				include_once $view_file;
			}
			include plugin_dir_path( WPADCENTER_PLUGIN_FILENAME ) . 'admin/views/admin-display-save-button.php';
			?>
		</div>
		<?php
	}

	/**
	 * Set impressions.
	 *
	 * @since 1.0.1
	 * @param int $banner_id Banner ID.
	 * @param int $banner_campaign Campaign ID.
	 * @param int $banner_advertiser Advertiser ID.
	 * @param int $banner_ad_zone Adzone ID.
	 */
	public static function wpadcenter_set_impressions( $banner_id, $banner_campaign, $banner_advertiser, $banner_ad_zone ) {
		global $wpdb;
		$today   = gmdate( 'Y-m-d' );
		$records = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM ' . $wpdb->prefix . 'adc_statistics WHERE banner_id = %d AND campaign_id = %d AND advertiser_id = %d AND adzone_id = %d AND date = %s LIMIT 1', array( $banner_id, $banner_campaign, $banner_advertiser, $banner_ad_zone, $today ) ), ARRAY_A ); // db call ok; no-cache ok.
		if ( count( $records ) ) {
			$record      = $records[0];
			$impressions = $record['impressions'] + 1;
			$wpdb->query( $wpdb->prepare( 'UPDATE ' . $wpdb->prefix . 'adc_statistics SET impressions = %d WHERE banner_id = %d AND campaign_id = %d AND advertiser_id = %d AND adzone_id = %d AND date = %s', array( $impressions, $banner_id, $banner_campaign, $banner_advertiser, $banner_ad_zone, $today ) ) ); // db call ok; no-cache ok.
			do_action( 'wpadcenter_after_set_impressions', $banner_id, $banner_campaign, $banner_advertiser, $banner_ad_zone, $impressions );
		} else {
			$wpdb->query( $wpdb->prepare( 'INSERT IGNORE INTO `' . $wpdb->prefix . 'adc_statistics` (`banner_id`,`campaign_id`,`advertiser_id`, `adzone_id`, `impressions`, `date`) VALUES (%d,%d,%d,%d,%d,%s)', array( $banner_id, $banner_campaign, $banner_advertiser, $banner_ad_zone, 1, $today ) ) ); // db call ok; no-cache ok.
		}
	}

	/**
	 * Connect to the filesystem.
	 *
	 * @param array $directories                  A list of directories. If any of these do
	 *                                            not exist, a WP_Error object will be returned.
	 * @return bool|WP_Error True if able to connect, false or a WP_Error otherwise.
	 */
	public static function fs_connect( $directories = array() ) {
		global $wp_filesystem;
		$directories = ( is_array( $directories ) && count( $directories ) ) ? $directories : array( WP_CONTENT_DIR );

		// This will output a credentials form in event of failure, We don't want that, so just hide with a buffer.
		ob_start();
		$credentials = request_filesystem_credentials( '', '', false, $directories[0] );
		ob_end_clean();

		if ( false === $credentials ) {
			return false;
		}

		if ( ! WP_Filesystem( $credentials ) ) {
			$error = true;
			if ( is_object( $wp_filesystem ) && $wp_filesystem->errors->get_error_code() ) {
				$error = $wp_filesystem->errors;
			}
			// Failed to connect, Error and request again.
			ob_start();
			request_filesystem_credentials( '', '', $error, $directories[0] );
			ob_end_clean();
			return false;
		}

		if ( ! is_object( $wp_filesystem ) ) {
			return new WP_Error( 'fs_unavailable', __( 'Could not access filesystem.', 'wpadcenter' ) );
		}

		if ( is_wp_error( $wp_filesystem->errors ) && $wp_filesystem->errors->get_error_code() ) {
			return new WP_Error( 'fs_error', __( 'Filesystem error.', 'wpadcenter' ), $wp_filesystem->errors );
		}

		foreach ( (array) $directories as $dir ) {
			switch ( $dir ) {
				case ABSPATH:
					if ( ! $wp_filesystem->abspath() ) {
						return new WP_Error( 'fs_no_root_dir', __( 'Unable to locate WordPress root directory.', 'wpadcenter' ) );
					}
					break;
				case WP_CONTENT_DIR:
					if ( ! $wp_filesystem->wp_content_dir() ) {
						return new WP_Error( 'fs_no_content_dir', __( 'Unable to locate WordPress content directory (wp-content).', 'wpadcenter' ) );
					}
					break;
				default:
					if ( ! $wp_filesystem->find_folder( $dir ) ) {
						return new WP_Error(
							'fs_no_folder',
							sprintf(
								/* translators: %s folder name */
								__( 'Unable to locate needed folder (%s).', 'wpadcenter' ),
								esc_html( basename( $dir ) )
							)
						);
					}
					break;
			}
		}

		return true;
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Wpadcenter_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
